# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2009 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "test_argument_without_name.hpp"
        self.config.cflags = "-std=c++11"

    def test_argument_without_name(self):

        """
        Test passing an object without name to a templated function.

        The test was failing when building the declaration string.
        The declaration string will be 'void (*)(  & )'. If the passed
        object had a name the result would then be 'void (*)(Name & )'.

        See bug #55

        """

        if self.config.xml_generator == "gccxml":
            return

        decls = parser.parse([self.header], self.config)
        global_ns = declarations.get_global_namespace(decls)

        criteria = declarations.calldef_matcher(name="function")
        free_funcs = declarations.matcher.find(criteria, global_ns)
        for free_func in free_funcs:
            decl_string = free_func.create_decl_string(with_defaults=False)
            self.assertEqual(decl_string, "void (*)(  & )")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser


class Test(parser_test_case.parser_test_case_t):

    def test(self):
        """
        Test different compilation standards by setting cflags.

        """

        # Skip this test for gccxml, this is a CastXML feature.
        if "gccxml" in self.config.xml_generator:
            return True

        parser.parse(["cpp_standards.hpp"], self.config)

        self.config.cflags = "-std=c++98"
        parser.parse(["cpp_standards.hpp"], self.config)

        self.config.cflags = "-std=c++03"
        parser.parse(["cpp_standards.hpp"], self.config)

        self.config.cflags = "-std=c++11"
        parser.parse(["cpp_standards.hpp"], self.config)

        # This is broken with llvm 3.6.2 (the one from homebrew)
        # It should work with never llvms but I keep the test disabled
        # See https://llvm.org/bugs/show_bug.cgi?id=24872
        # self.config.cflags = "-std=c++14"
        # parser.parse(["cpp_standards.hpp"], self.config)

        # Same as above
        # self.config.cflags = "-std=c++1z"
        # parser.parse(["cpp_standards.hpp"], self.config)

        # Pass down a flag that does not exist.
        # This should raise an exception.
        self.config.cflags = "-std=c++00"
        self.assertRaises(
            RuntimeError,
            lambda: parser.parse(["cpp_standards.hpp"], self.config))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import parser_test_case

from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):

    def test(self):
        path = os.path.normpath("/mypath/folder1/folder2/folder3")
        dirs = [
            "/mypath/folder1/folder2/",
            "/mypath3/folder1/folder2/folder3",
            "home",
            "/test/test1/mypath"]

        self.assertTrue(utils.utils.contains_parent_dir(path, dirs))

        dirs = ["/home", "/mypath/test/"]

        self.assertFalse(utils.utils.contains_parent_dir(path, dirs))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import sys
import logging
import warnings

# Prevents copy.deepcopy RecursionError in some tests (Travis build)
sys.setrecursionlimit(10000)

this_module_dir_path = os.path.abspath(
    os.path.dirname(sys.modules[__name__].__file__))

data_directory = os.path.join(this_module_dir_path, 'data')
build_directory = os.path.join(this_module_dir_path, 'temp')

sys.path.insert(1, os.path.join(os.curdir, '..'))
# The tests are run on the parent pygccxml directory, not the one
# in site-packages. Insert the directory's path.
sys.path.insert(1, "../pygccxml")

import pygccxml  # nopep8
import pygccxml.declarations  # nopep8
import pygccxml.parser  # nopep8
import pygccxml.utils  # nopep8

# We want to make sure we throw an error for ALL the warnings during the
# tests. This will allow us to be notified by the build bots, so that the
# warnings can be fixed.
warnings.simplefilter("error", Warning)

# Set logging level
pygccxml.utils.loggers.set_level(logging.INFO)

# Find out the c++ parser (gccxml or castxml)
generator_path, generator_name = pygccxml.utils.find_xml_generator()

pygccxml.declarations.class_t.USE_DEMANGLED_AS_NAME = True


class cxx_parsers_cfg(object):
    gccxml = pygccxml.parser.load_xml_generator_configuration(
        'xml_generator.cfg',
        xml_generator_path=generator_path,
        working_directory=data_directory,
        compiler=None,
        xml_generator=generator_name)

    if generator_name == 'gccxml':
        gccxml.define_symbols.append('__GCCXML_09__')

    if 'nt' == os.name:
        gccxml.define_symbols.append(
            '__PYGCCXML_%s__' %
            gccxml.compiler.upper())
        if 'msvc9' == gccxml.compiler:
            gccxml.define_symbols.append('_HAS_TR1=0')

if cxx_parsers_cfg.gccxml.xml_generator:
    generator_name = cxx_parsers_cfg.gccxml.xml_generator
if cxx_parsers_cfg.gccxml.xml_generator_path:
    generator_path = cxx_parsers_cfg.gccxml.xml_generator_path

print(
    '%s configured to simulate compiler %s' %
    (generator_name.title(), cxx_parsers_cfg.gccxml.compiler))



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)

    def test(self):
        fconfig = parser.file_configuration_t(
            data='int i;',
            start_with_declarations=None,
            content_type=parser.file_configuration_t.CONTENT_TYPE.TEXT)

        prj_reader = parser.project_reader_t(self.config)
        decls = prj_reader.read_files(
            [fconfig],
            compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)

        var_i = declarations.find_declaration(
            decls, decl_type=declarations.variable_t, name='i')
        self.assertTrue(var_i, "Variable i has not been found.")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)

    def test1(self):
        code = 'int aaaa[2][3][4][5];'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        aaaa_type = global_ns.variable('aaaa').decl_type
        self.assertTrue(
            'int[2][3][4][5]' == aaaa_type.decl_string,
            aaaa_type.decl_string)

    def test2(self):
        code = 'int* aaaa[2][3][4][5];'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        aaaa_type = global_ns.variable('aaaa').decl_type
        self.assertTrue(
            'int *[2][3][4][5]' == aaaa_type.decl_string,
            aaaa_type.decl_string)

    def test3(self):
        code = 'int aaaa[2];'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        aaaa_type = global_ns.variable('aaaa').decl_type
        self.assertTrue(
            'int[2]' == aaaa_type.decl_string,
            aaaa_type.decl_string)

    def test4(self):
        code = 'struct xyz{}; xyz aaaa[2][3];'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        aaaa_type = global_ns.variable('aaaa').decl_type
        self.assertTrue(
            '::xyz[2][3]' == aaaa_type.decl_string,
            aaaa_type.decl_string)

    def test5(self):
        code = 'char const arr[4] = {};'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        arr_type = global_ns.variable('arr').decl_type
        if self.config.xml_generator == "gccxml":
            self.assertTrue(
                'char[4] const' == arr_type.decl_string,
                arr_type.decl_string)
        else:
            self.assertTrue(
                'char const[4]' == arr_type.decl_string,
                arr_type.decl_string)
        self.assertTrue(
            declarations.is_array(arr_type))
        self.assertTrue(
            declarations.is_const(arr_type))

    def test6(self):
        code = 'char volatile arr[4] = {};'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        arr_type = global_ns.variable('arr').decl_type
        if self.config.xml_generator == "gccxml":
            self.assertTrue(
                'char[4] volatile' == arr_type.decl_string,
                arr_type.decl_string)
        else:
            self.assertTrue(
                'char volatile[4]' == arr_type.decl_string,
                arr_type.decl_string)
        self.assertTrue(
            declarations.is_array(arr_type))
        self.assertTrue(
            declarations.is_volatile(arr_type))

    def test7(self):
        code = 'char const volatile arr[4] = {};'
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        arr_type = global_ns.variable('arr').decl_type
        if self.config.xml_generator == "gccxml":
            self.assertTrue(
                'char[4] const volatile' == arr_type.decl_string,
                arr_type.decl_string)
        else:
            self.assertTrue(
                'char const volatile[4]' == arr_type.decl_string,
                arr_type.decl_string)
        self.assertTrue(
            declarations.is_array(arr_type))
        self.assertTrue(
            declarations.is_const(arr_type))
        self.assertTrue(
            declarations.is_volatile(arr_type))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import sys
from platform import system
import unittest
import autoconfig
import parser_test_case
import subprocess
from pygccxml import binary_parsers
from pygccxml import utils
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    global_ns = None

    @property
    def known_issues(self):
        if 'nt' == os.name:
            issues = set([
                # array as function argument: 'int FA10_i_i(int * const)'
                # pointer to function: 'void myclass_t::set_do_smth(void
                # (**)(std::auto_ptr<number_t> &))'
                # pointer to functions: 'void (**
                # myclass_t::get_do_smth(void))(std::auto_ptr<number_t> &)'
                '?FA10_i_i@@YAHQAH@Z',
                ('?set_do_smth@myclass_t@@QAEXPAP6AXAAV?' +
                    '$auto_ptr@Vnumber_t@@@std@@@Z@Z'),
                ('?get_do_smth@myclass_t@@QAEPAP6AXAAV?' +
                    '$auto_ptr@Vnumber_t@@@std@@@ZXZ')
            ])
            if 'msvc71' == utils.native_compiler.get_gccxml_compiler():
                # missing reference in argument - compiler issue
                # ('std::auto_ptr<number_t> &
                # std::auto_ptr<number_t>::operator=' +
                # '(std::auto_ptr_ref<number_t>)')
                issues.add((
                    '??4?$auto_ptr@Vnumber_t@@@std@@QAEAAV01@U?' +
                    '$auto_ptr_ref@Vnumber_t@@@1@@Z'))
            return issues
        else:
            return set([
                # even c++filt fails
                # typeinfo name for number_t
                # it seems that gccxml doesn't report this one
                # the following are some global symbols
                '_ZNSt8auto_ptrI8number_tEcvSt12auto_ptr_refIT_EIS0_EEv',
                '_ZTI8number_t', '_ZTV8number_t', '_ZTS8number_t',
                '_ZNSt12auto_ptr_refI8number_tEC1EPS0_', '_init', '_edata',
                '_fini', '_end'])

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.binary_parsers_dir = os.path.join(
            autoconfig.data_directory,
            'binary_parsers')
        self.header = os.path.join(self.binary_parsers_dir, r'mydll.h')
        self.map_file = os.path.join(
            self.binary_parsers_dir,
            'binaries',
            'mydll.map')
        self.dll_file = os.path.join(
            self.binary_parsers_dir,
            'binaries',
            'mydll.dll')
        system_name = system()
        if system_name == 'Darwin':
            ext = '.dylib'
        elif system_name == 'Windows':
            ext = '.dll'
        else:
            ext = '.so'
        self.so_file = os.path.join(
            self.binary_parsers_dir,
            'binaries',
            'libmydll%s' %
            ext)

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

            process = subprocess.Popen(
                args='scons msvc_compiler=%s' %
                autoconfig.cxx_parsers_cfg.gccxml.compiler,
                shell=True,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                cwd=self.binary_parsers_dir)
            process.stdin.close()

            while process.poll() is None:
                line = process.stdout.readline()
                print(line.rstrip())
            for line in process.stdout.readlines():
                print(line.rstrip())
            if process.returncode:
                raise RuntimeError(
                    ("unable to compile binary parser module. " +
                        "See output for the errors."))

    def is_included(self, decl):
        if not isinstance(
                decl, (declarations.calldef_t, declarations.variable_t)):
            return False
        for suffix in [self.header, 'memory']:
            if decl.location.file_name.endswith(suffix):
                return True
        else:
            return False

    def __tester_impl(self, fname, expected_symbols):
        symbols, parser = binary_parsers.merge_information(
            self.global_ns, fname, runs_under_unittest=True)
        # this doesn't work reliably
        # self.assertTrue(
        #    len(symbols) == expected_symbols,
        # "The expected symbols number(%d),
        # is different from the actual one(%d)"
        # % ( expected_symbols, len(symbols) ) )
        self.assertTrue('identity' in symbols)

        msg = []
        blob_names = set()
        for blob in parser.loaded_symbols:
            if isinstance(blob, tuple):
                blob = blob[0]
            if 'nt' == os.name:
                # TODO: find out where undecorate function is exposed on linux
                undname = binary_parsers.undecorate_blob(blob)
                if "`" in undname:
                    continue
            blob_names.add(blob)

        decl_blob_names = set(symbols.keys())

        issuperset = decl_blob_names.issuperset(blob_names)
        if not issuperset:
            common = decl_blob_names.intersection(blob_names)

            decl_blob_names.difference_update(common)
            blob_names.difference_update(common)
            if not self.known_issues.issubset(blob_names):
                blob_names.difference_update(self.known_issues)
                if sys.version_info[0] == 2 and sys.version_info[1] == 5:
                    if 0 == len(decl_blob_names) and 0 == len(blob_names):
                        return
                msg.append("decl_blob_names :")
                for i in decl_blob_names:
                    msg.append('\t==>%s<==' % i)
                msg.append("blob_names :")
                for i in blob_names:
                    msg.append('\t==>%s<==' % i)

                self.fail(os.linesep.join(msg))

    def test_map_file(self):
        if 'nt' == os.name:
            self.__tester_impl(self.map_file, 71)

    def test_dll_file(self):
        if 'nt' == os.name:
            self.__tester_impl(self.dll_file, 71)

    def test_z_compare_parsers(self):
        if 'nt' != os.name:
            return
        dsymbols, dparser = binary_parsers.merge_information(
            self.global_ns, self.dll_file, runs_under_unittest=True)
        msymbols, mparser = binary_parsers.merge_information(
            self.global_ns, self.map_file, runs_under_unittest=True)

        self.assertTrue(
            len(dparser.loaded_symbols) == len(mparser.loaded_symbols))

        was_error = False
        for blob, decl in dsymbols.items():
            if blob not in msymbols:
                was_error = True
                print(
                    '\n%s could not be found in .map file' %
                    binary_parsers.undecorate_blob(blob))
                # self.assertTrue( blob in msymbols,
                # binary_parsers.undecorate_blob( blob ) )
            else:
                mdecl = msymbols[blob]
                self.assertTrue(mdecl is decl)
        self.assertTrue(was_error is False)

    def test_so_file(self):
        if 'posix' in os.name:
            self.__tester_impl(self.so_file, 64)

    def dont_test_print(self):
        """primary used for debugging"""
        symbols, parser = binary_parsers.merge_information(
            self.global_ns, self.so_file, runs_under_unittest=True)
        for f in self.global_ns.calldefs(allow_empty=True, recursive=True):
            print(binary_parsers.format_decl(f, 'nm'))

        for v in self.global_ns.variables(allow_empty=True, recursive=True):
            print(binary_parsers.format_decl(v, 'nm'))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""Python CastXML or GCC-XML front end.

This package provides functionality to extract and inspect
declarations from C/C++ header files. This is accomplished
by invoking an external tool like CastXML or GCC-XML,
which parses a header file and dumps the declarations as a
XML file. This XML file is then read by pygccxml and the contents
are made available as appropriate Python objects.

To parse a set of C/C++ header files you use the :func:`parse <parser.parse>`
function in the :mod:parser sub package which returns a tree that contains all
declarations found in the header files. The root of the tree represents the
main namespace `::` and the children nodes represent the namespace contents
such as other namespaces, classes, functions, etc. Each node in the tree is an
object of a type derived from the :class:`declaration_t` class. An inner node
is always either a namespace :class:`declarations.namespace_t` or a class
:class:`declarations.class_t`, which are both derived from
:class:`declarations.scopedef_t` class. Everything else (free functions,
member functions, enumerations, variables, etc.) are always a leaf. You will
find all those declaration classes in the :mod:declarations sub-package.

"""

from . import declarations
from . import parser
from . import utils

# Always show deprecation warnings.
# These are hidden by default since python 2.7.
# pygccxml is a tool for developers, and these need
# to know what is deprecated.
import warnings
warnings.simplefilter("always", DeprecationWarning)

# TODO:
# 1. Add "explicit" property for constructors

__version__ = '1.8.0'



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import autoconfig
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class tester_impl_t(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = os.path.join(
            autoconfig.data_directory,
            'declarations_enums.hpp')
        self.cache_file = os.path.join(
            autoconfig.data_directory,
            'pygccxml.cache')
        if os.path.exists(self.cache_file) and os.path.isfile(self.cache_file):
            os.remove(self.cache_file)

    def test_cache(self):
        cache = parser.file_cache_t(self.cache_file)
        reader = parser.source_reader_t(self.config, cache)
        decls1 = reader.read_file(self.header)
        cache.flush()
        cache = parser.file_cache_t(self.cache_file)
        reader = parser.source_reader_t(self.config, cache)
        decls2 = reader.read_file(self.header)

        enum_matcher = declarations.declaration_matcher_t(
            name="EColor",
            decl_type=declarations.enumeration_t)

        color1 = declarations.matcher.get_single(enum_matcher, decls1)
        color2 = declarations.matcher.get_single(enum_matcher, decls2)
        self.assertTrue(color1.values == color2.values)

# there is no progress with this parser
# class synopsis_tester_t( tester_impl_t ):
#    CXX_PARSER_CFG = autoconfig.cxx_parsers_cfg.synopsis


class gccxml_tester_t(tester_impl_t):
    CXX_PARSER_CFG = autoconfig.cxx_parsers_cfg.gccxml


def create_suite():
    suite = unittest.TestSuite()
    # suite.addTest( unittest.makeSuite(synopsis_tester_t))
    suite.addTest(unittest.makeSuite(gccxml_tester_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import sys
import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__files = [
            'core_ns_join_1.hpp',
            'core_ns_join_2.hpp',
            'core_ns_join_3.hpp',
            'core_membership.hpp',
            'core_class_hierarchy.hpp',
            'core_types.hpp',
            'core_diamand_hierarchy_base.hpp',
            'core_diamand_hierarchy_derived1.hpp',
            'core_diamand_hierarchy_derived2.hpp',
            'core_diamand_hierarchy_final_derived.hpp',
            'core_overloads_1.hpp',
            'core_overloads_2.hpp',
            'typedefs_base.hpp']

        # for i, f in enumerate(self.__files):
        # f = parser.create_cached_source_fc(
        #   os.path.join( autoconfig.data_directory, f)
        # , os.path.join( autoconfig.data_directory, f + '.xml') )
        # self.__files[i] = f
        prj_reader = parser.project_reader_t(self.config)
        self.decls = prj_reader.read_files(
            self.__files,
            compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)

    def test_printer(self):

        # Redirect sys.stdout to a class with a writer doing nothing
        # This greatly reduces the size of the test output and makes
        # test log files readable.
        # Note: flush needs to be defined; because if not this will
        # result in an AttributeError on call.
        class DontPrint(object):
            def write(*args):
                pass

            def flush(*args):
                pass
        sys.stdout = DontPrint()

        declarations.print_declarations(self.decls, writer=None)

    def test__str__(self):
        decls = declarations.make_flatten(self.decls)
        for decl in decls:
            str(decl)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import hotshot
import hotshot.stats

import autoconfig
import test_parser

if __name__ == "__main__":
    statistics_file = os.path.join(autoconfig.data_directory, 'profile.stat')
    profile = hotshot.Profile(statistics_file)
    profile.runcall(test_parser.run_suite)
    profile.close()
    statistics = hotshot.stats.load(statistics_file)
    statistics.strip_dirs()
    statistics.sort_stats('time', 'calls')
    statistics.print_stats(20)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import autoconfig
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__files = [
            'core_types.hpp',
            'core_ns_join_1.hpp',
            'core_ns_join_2.hpp',
            'core_ns_join_3.hpp',
            'core_membership.hpp',
            'core_class_hierarchy.hpp',
            'core_diamand_hierarchy_base.hpp',
            'core_diamand_hierarchy_derived1.hpp',
            'core_diamand_hierarchy_derived2.hpp',
            'core_diamand_hierarchy_final_derived.hpp',
            'core_overloads_1.hpp',
            'core_overloads_2.hpp']

    def __test_correctness_impl(self, file_name):
        prj_reader = parser.project_reader_t(self.config)
        prj_decls = prj_reader.read_files(
            [file_name] * 2,
            compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)
        src_reader = parser.source_reader_t(self.config)
        src_decls = src_reader.read_file(file_name)
        if src_decls != prj_decls:
            s = src_decls[0]
            p = prj_decls[0]
            bdir = autoconfig.build_directory
            with open(os.path.join(bdir, file_name + '.sr.txt'), 'w+') as sr:
                with open(
                        os.path.join(bdir, file_name + '.pr.txt'), 'w+') as pr:

                    declarations.print_declarations(
                        s, writer=lambda l: sr.write(l + os.linesep))
                    declarations.print_declarations(
                        p, writer=lambda l: pr.write(l + os.linesep))

            self.fail(
                "There is a difference between declarations in file %s." %
                file_name)

    def test_correctness(self):
        for src in self.__files:
            self.__test_correctness_impl(src)


class tester2_t(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__files = [
            'separate_compilation/data.h',
            'separate_compilation/base.h',
            'separate_compilation/derived.h']

    def test(self):
        prj_reader = parser.project_reader_t(self.config)
        prj_decls = prj_reader.read_files(
            self.__files,
            compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)
        src_reader = parser.source_reader_t(self.config)
        src_decls = src_reader.read_file('separate_compilation/all.h')

        declarations.dump_declarations(
            src_decls,
            os.path.join(
                autoconfig.build_directory, 'separate_compilation.sr.txt'))

        declarations.dump_declarations(
            prj_decls,
            os.path.join(
                autoconfig.build_directory, 'separate_compilation.pr.txt'))

        self.assertTrue(
            src_decls == prj_decls,
            "There is a difference between declarations")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    suite.addTest(unittest.makeSuite(tester2_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""
defines class that describes C++ typedef declaration
"""

import warnings
from . import declaration
from . import class_declaration


class typedef_t(declaration.declaration_t):

    """describes C++ typedef declaration"""

    def __init__(self, name='', type=None, decl_type=None):
        """creates class that describes C++ typedef"""

        if type is not None:
            warnings.warn(
                "The type argument is deprecated. \n" +
                "Please use the decl_type argument instead.",
                DeprecationWarning)
            if decl_type is not None:
                raise (
                    "Please use only either the type or " +
                    "decl_type argument.")
            # Still allow to use the old type for the moment.
            decl_type = type

        declaration.declaration_t.__init__(self, name)
        self._decl_type = decl_type

    def _get__cmp__items(self):
        """implementation details"""
        return [self.decl_type]

    def __eq__(self, other):
        if not declaration.declaration_t.__eq__(self, other):
            return False
        return self.decl_type == other.decl_type

    def __hash__(self):
        return super.__hash__(self)

    @property
    def type(self):
        """
        Deprecated since v1.8.0. Will be removed in v1.9.0

        """

        warnings.warn(
            "typedef_t.type is deprecated.\n" +
            "Please use typedef_t.decl_type instead.", DeprecationWarning)
        return self._decl_type

    @type.setter
    def type(self, _decl_type):
        """
        Deprecated since v1.8.0. Will be removed in v1.9.0

        """

        warnings.warn(
            "typedef_t.type is deprecated.\n" +
            "Please use typedef_t.decl_type instead.", DeprecationWarning)
        self._decl_type = _decl_type

    @property
    def decl_type(self):
        """reference to the original :class:`decl_type <type_t>`"""
        return self._decl_type

    @decl_type.setter
    def decl_type(self, decl_type):
        self._decl_type = decl_type

    def i_depend_on_them(self, recursive=True):
        return [class_declaration.dependency_info_t(self, self.decl_type)]

    @property
    def byte_size(self):
        """Size of this type in bytes @type: int"""
        return self._decl_type.byte_size

    @property
    def byte_align(self):
        """alignment of this type in bytes @type: int"""
        return self._decl_type.byte_align



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import pprint
import sys
import time
import unittest
import autoconfig


class parser_test_case_t(unittest.TestCase):

    CXX_PARSER_CFG = None

    def __init__(self, *args):
        unittest.TestCase.__init__(self, *args)
        if self.CXX_PARSER_CFG:
            self.config = self.CXX_PARSER_CFG.clone()
        elif autoconfig.cxx_parsers_cfg.gccxml:
            self.config = autoconfig.cxx_parsers_cfg.gccxml.clone()
        else:
            pass

    def run(self, result=None):
        """
        Override the run method.

        Allows to measure the time each test needs. The result is written
        in the test_cost.log file.

        """
        with open("test_cost.log", "a") as cost_file:
            start_time = time.time()
            super(parser_test_case_t, self).run(result)
            name = super(parser_test_case_t, self).id()
            cost_file.write(
                name + " " +
                str(time.time() - start_time) + "\n")

    def _test_type_composition(self, type_, expected_compound, expected_base):
        self.assertTrue(
            isinstance(type_, expected_compound),
            "the compound type('%s') should be '%s'" %
            (type_.decl_string, expected_compound.__name__))
        self.assertTrue(
            isinstance(type_.base, expected_base),
            "base type('%s') should be '%s'" %
            (type_.decl_string, expected_base.__name__))

    def _test_calldef_return_type(self, calldef, expected_type):
        self.assertTrue(
            isinstance(calldef.return_type, expected_type),
            ("the function's '%s' expected return type is '%s' and in " +
                "reality it is different('%s')") %
            (calldef.name, expected_type.__name__,
             calldef.return_type.__class__.__name__))

    def _test_calldef_args(self, calldef, expected_args):
        self.assertTrue(
            len(calldef.arguments) == len(expected_args),
            ("the function's '%s' expected number of arguments is '%d' and " +
                "in reality it is different('%d')") %
            (calldef.name, len(expected_args), len(calldef.arguments)))

        for i, expected_arg in enumerate(expected_args):
            arg = calldef.arguments[i]
            self.assertTrue(
                arg == expected_arg,
                ("the function's '%s' expected %d's argument is '%s' and in " +
                    "reality it is different('%s')") %
                (calldef.name, i, pprint.pformat(expected_arg.__dict__),
                 pprint.pformat(arg.__dict__)))

    def _test_calldef_exceptions(self, calldef, exceptions):
        # exceptions is list of classes names
        exception_decls = []
        for name in exceptions:
            exception_decl = self.global_ns.class_(name)
            self.assertTrue(
                exception_decl,
                "unable to find exception class '%s'" %
                name)
            exception_decls.append(exception_decl)
        exception_decls.sort()
        self.assertTrue(
            len(calldef.exceptions) == len(exception_decls),
            ("the function's '%s' expected number of exceptions is '%d' and " +
                "in reality it is different('%d')") %
            (calldef.name,
             len(exception_decls),
             len(calldef.exceptions)))
        exceptions_indeed = sorted(calldef.exceptions[:])
        self.assertTrue(
            exception_decls == exceptions_indeed,
            ("the function's '%s' expected exceptions are '%s' and in " +
                "reality it is different('%s')") %
            (calldef.name,
             pprint.pformat([delc.name for delc in exception_decls]),
             pprint.pformat([delc.name for delc in exceptions_indeed])))

if sys.version_info < (2, 7, 0):
    # Python2.6 does not have the following methods in the unittest module
    parser_test_case_t.assertIn = \
        lambda parser, a1, a2, *args: parser.assertTrue(a1 in a2, args)
    parser_test_case_t.assertNotIn = \
        lambda parser, a1, a2, *args: parser.assertFalse(a1 in a2, args)
    parser_test_case_t.assertIs = \
        lambda parser, a1, a2, *args: parser.assertTrue(a1 is a2, args)
    parser_test_case_t.assertIsNot = \
        lambda parser, a1, a2, *args: parser.assertFalse(a1 is a2, args)
    parser_test_case_t.assertIsNone = \
        lambda parser, a1, *args: parser.assertTrue(a1 is None, args)
    parser_test_case_t.assertIsNotNone = \
        lambda parser, a1, *args: parser.assertFalse(a1 is None, args)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt


"""
defines few simple classes( parsers ), which deals with .dll, .map, .so files.

Those classes extract decorated\mangled names from the files. Later the
extracted symbols are used for:

   * building "dynamic library" public interface
   * extracting function calling convention

"""

import os
import re
import platform
from . import undname
import warnings
import subprocess
from .. import declarations

# import exceptions
# class LicenseWarning( exceptions.UserWarning ):
#     def __init__( self, *args, **keywd ):
#         exceptions.UserWarning.__init__( self, *args, **keywd )


dll_file_parser_warning = \
    """
----------------------------->> LICENSE  WARNING <<----------------------------
"dll_file_parser_t" functionality uses code licensed under MIT license.
pygccxml project uses Boost Software License, Version 1.0.
For more information about this functionality take a look on
get_dll_exported_symbols.py file.
===============================================================================

"""


class libparser_t(object):

    """base class for .dll, .map, .so parser classes"""

    def __init__(self, global_ns, binary_file):
        """global_ns - reference to global namespace
        binary_file - s
        """
        self.__global_ns = global_ns
        self.__binary_file = binary_file
        self.__loaded_symbols = None

    @property
    def global_ns(self):
        """reference to global namespace"""
        return self.__global_ns

    @property
    def binary_file(self):
        """binary file path"""
        return self.__binary_file

    @property
    def loaded_symbols(self):
        """list of symbols, which were loaded from the binary file.
        The actual type of return value defined by the derived class"""
        return self.__loaded_symbols

    def load_symbols(self):
        """loads public( shared ) symbols from the binary file.

        This method should be overridden in the derived classes.
        """
        raise NotImplementedError()

    def merge(self, symbol):
        """extracts and merges information from the symbol to the declarations
        tree.

        This method should be overridden in the derived classes.
        """
        raise NotImplementedError()

    def parse(self):
        """the main method of the class

        loads information from the binary file and merges it into the
        declarations tree.

        The return value of the function is dictionary, where the key is
        decorated/mangled declaration name and the value is a declaration.
        """
        self.__loaded_symbols = self.load_symbols()
        result = {}
        for smbl in self.__loaded_symbols:
            try:
                decorated, decl = self.merge(smbl)
                if decl:
                    decl.decorated_name = decorated
                    result[decorated] = decl
            except declarations.matcher.declaration_not_found_t:
                pass
        return result

CCTS = declarations.CALLING_CONVENTION_TYPES


class formated_mapping_parser_t(libparser_t):

    """convenience class, which formats existing declarations"""

    def __init__(self, global_ns, binary_file, hint):
        libparser_t.__init__(self, global_ns, binary_file)
        self.__formated_decls = {}
        self.undname_creator = undname.undname_creator_t()

        for f in self.global_ns.calldefs(allow_empty=True, recursive=True):
            self.__formated_decls[self.formatter(f, hint)] = f

        for v in self.global_ns.variables(allow_empty=True, recursive=True):
            self.__formated_decls[self.formatter(v, hint)] = v

    @property
    def formated_decls(self):
        return self.__formated_decls

    def formatter(self, decl, hint):
        return self.undname_creator.format_decl(decl, hint)


class map_file_parser_t(formated_mapping_parser_t):

    """parser for MSVC .map file"""
    c_entry = re.compile(
        r' +\d+    (?P<internall>.+?)(?:\s+exported name\:\s(?P<name>.*)$)')
    cpp_entry = re.compile(
        r' +\d+    (?P<decorated>.+?) \((?P<undecorated>.+)\)$')

    def __init__(self, global_ns, map_file_path):
        formated_mapping_parser_t.__init__(
            self, global_ns, map_file_path, 'msvc')

    def load_symbols(self):
        """returns dictionary
        { decorated symbol : original declaration name }"""
        with open(self.binary_file) as f:
            lines = []
            was_exports = False
            for line in f:
                if was_exports:
                    lines.append(line)
                elif 'Exports' == line.strip():
                    was_exports = True
                else:
                    pass

        index = 0
        result = []
        while index < len(lines):
            line = lines[index].rstrip()
            found = self.cpp_entry.match(line.decode('ascii'))
            if found:
                result.append(
                    (found.group('decorated'),
                     found.group('undecorated')))
            elif index + 1 < len(lines):
                two_lines = line + lines[index + 1].rstrip()
                found = self.c_entry.match(two_lines)
                if found:
                    result.append((found.group('name'), found.group('name')))
                    index += 1
            else:
                pass
            index += 1
        return result

    def merge(self, smbl):
        decorated, undecorated = smbl
        if decorated == undecorated:
            # we deal with C function ( or may be we deal with variable?, I
            # have to check the latest
            f = self.global_ns.free_fun(decorated)
            # TODO create usecase, where C function uses different calling
            # convention
            f.calling_convention = CCTS.CDECL
            return decorated, f
        else:
            undecorated_normalized = \
                self.undname_creator.normalize_undecorated(undecorated)
            if undecorated_normalized not in self.formated_decls:
                return None, None
            decl = self.formated_decls[undecorated_normalized]
            if isinstance(decl, declarations.calldef_t):
                decl.calling_convention = CCTS.extract(undecorated, CCTS.CDECL)
            return decorated, decl


class dll_file_parser_t(formated_mapping_parser_t):

    """parser for Windows .dll file"""

    def __init__(self, global_ns, map_file_path):
        warnings.warn(dll_file_parser_warning, LicenseWarning)
        formated_mapping_parser_t.__init__(
            self,
            global_ns,
            map_file_path,
            'msvc')

    def load_symbols(self):
        from . import get_dll_exported_symbols
        return get_dll_exported_symbols.read_export_table(self.binary_file)

    def merge(self, smbl):
        blob = smbl
        blob_undecorated = self.undname_creator.undecorate_blob(
            blob,
            undname.UNDECORATE_NAME_OPTIONS.UNDNAME_COMPLETE)
        blob_undecorated_normalized = self.undname_creator.undecorate_blob(
            blob)
        if blob == blob_undecorated == blob_undecorated_normalized:
            # we deal with C function ( or may be we deal with variable?, I
            # have to check the latest
            f = self.global_ns.free_fun(blob)
            # TODO create usecase, where C function uses different calling
            # convention
            f.calling_convention = CCTS.CDECL
            return blob, f
        else:
            if blob_undecorated_normalized not in self.formated_decls:
                return None, None
            decl = self.formated_decls[blob_undecorated_normalized]
            if isinstance(decl, declarations.calldef_t):
                decl.calling_convention = CCTS.extract(
                    blob_undecorated,
                    CCTS.CDECL)
            return blob, decl


class so_file_parser_t(formated_mapping_parser_t):

    """parser for Linux .so file"""
    nm_executable = 'nm'
    # numeric-sort used for mapping between mangled and unmangled name
    cmd_mangled = (
        '%(nm)s --extern-only --dynamic --defined-only ' +
        '--numeric-sort %(lib)s').split()
    cmd_demangled = (
        '%(nm)s --extern-only --dynamic --defined-only ' +
        '--demangle --numeric-sort %(lib)s').split()

    entry = re.compile(r'^(?P<address>(?:\w|\d)+)\s\w\s(?P<symbol>.+)$')

    def __init__(self, global_ns, binary_file):
        formated_mapping_parser_t.__init__(self, global_ns, binary_file, 'nm')

    def __execute_nm(self, cmd):
        process = subprocess.Popen(
            args=cmd,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)
        process.stdin.close()

        output = []
        while process.poll() is None:
            output.append(process.stdout.readline())
        # the process already finished, read th rest of the output
        output.extend(process.stdout.readlines())
        if process.returncode:
            msg = [(
                "Unable to extract public\\exported symbols from '%s' file.") %
                self.binary_file]
            msg.append(
                ('The command line, which was used to extract symbols, ' +
                    'is "%s"') % cmd)
            raise RuntimeError(os.linesep.join(msg))
        return output

    def __extract_symbols(self, cmd):
        output = self.__execute_nm(cmd)
        result = {}
        for line in output:
            found = self.entry.match(line)
            if found:
                result[found.group('address')] = found.group('symbol')
        return result

    def load_symbols(self):
        tmpl_args = dict(nm=self.nm_executable, lib=self.binary_file)
        mangled_smbls = self.__extract_symbols(
            [part % tmpl_args for part in self.cmd_mangled])
        demangled_smbls = self.__extract_symbols(
            [part % tmpl_args for part in self.cmd_demangled])

        result = []
        for address, blob in mangled_smbls.items():
            if address in demangled_smbls:
                result.append((blob, demangled_smbls[address]))
        return result

    def merge(self, smbl):
        decorated, undecorated = smbl
        if decorated == undecorated:
            # we deal with C function ( or may be we deal with variable?, I
            # have to check the latest
            try:
                f = self.global_ns.free_fun(decorated)
                # TODO create usecase, where C function uses different calling
                # convention
                f.calling_convention = CCTS.CDECL
                return decorated, f
            except self.global_ns.declaration_not_found_t:
                v = self.global_ns.variables(
                    decorated,
                    allow_empty=True,
                    recursive=False)
                if v:
                    return decorated, v[0]
                else:
                    return None, None
        else:
            undecorated_normalized = \
                self.undname_creator.normalize_undecorated(undecorated)
            if undecorated_normalized not in self.formated_decls:
                return None, None
            decl = self.formated_decls[undecorated_normalized]
            if isinstance(decl, declarations.calldef_t):
                decl.calling_convention = CCTS.extract(undecorated, CCTS.CDECL)
            return decorated, decl


class dylib_file_parser_t(formated_mapping_parser_t):

    """parser for Darwin .dylib file"""
    nm_executable = 'nm'
    # numeric-sort used for mapping between mangled and unmangled name
    cmd_mangled = ['-c', '%(nm)s -gfUn %(lib)s | sed "s/ _/ /"']
    cmd_demangled = ['-c', '%(nm)s -gfUn %(lib)s | sed "s/ _/ /" | c++filt']

    entry = re.compile(r'^(?P<address>(?:\w|\d)+)\s\w\s(?P<symbol>.+)$')

    def __init__(self, global_ns, binary_file):
        formated_mapping_parser_t.__init__(self, global_ns, binary_file, 'nm')

    def __execute_nm(self, cmd):
        process = subprocess.Popen(
            args=cmd,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            shell=True)
        process.stdin.close()
        output = []
        while process.poll() is None:
            output.append(process.stdout.readline())
        # the process already finished, read the rest of the output
        output.extend(process.stdout.readlines())
        if process.returncode:
            msg = [(
                "Unable to extract public/exported symbols from" +
                "'%s' file.") % self.binary_file]
            msg.append(
                ('The command line, which was used to extract symbols, ' +
                    'is "%s"') % cmd)
            raise RuntimeError(os.linesep.join(msg))
        return output

    def __extract_symbols(self, cmd):
        output = self.__execute_nm(cmd)
        result = {}
        for line in output:
            found = self.entry.match(str(line))
            if found:
                result[found.group('address')] = found.group('symbol')
        return result

    def load_symbols(self):
        tmpl_args = dict(nm=self.nm_executable, lib=self.binary_file)
        mangled_smbls = self.__extract_symbols(
            [part % tmpl_args for part in self.cmd_mangled])
        demangled_smbls = self.__extract_symbols(
            [part % tmpl_args for part in self.cmd_demangled])

        result = []
        for address, blob in mangled_smbls.items():
            if address in demangled_smbls:
                result.append((blob, demangled_smbls[address]))
        return result

    def merge(self, smbl):
        decorated, undecorated = smbl
        if decorated == undecorated:
            # we deal with C function ( or may be we deal with variable?, I
            # have to check the latest
            try:
                f = self.global_ns.free_fun(decorated)
                # TODO create usecase, where C function uses different calling
                # convention
                f.calling_convention = CCTS.CDECL
                return decorated, f
            except self.global_ns.declaration_not_found_t:
                v = self.global_ns.variables(
                    decorated,
                    allow_empty=True,
                    recursive=False)
                if v:
                    return decorated, v[0]
                else:
                    return None, None
        else:
            undecorated_normalized = \
                self.undname_creator.normalize_undecorated(undecorated)
            if undecorated_normalized not in self.formated_decls:
                return None, None
            decl = self.formated_decls[undecorated_normalized]
            if isinstance(decl, declarations.calldef_t):
                decl.calling_convention = CCTS.extract(undecorated, CCTS.CDECL)
            return decorated, decl


def merge_information(global_ns, fname, runs_under_unittest=False):

    """high level function - select the appropriate binary file parser and
    integrates the information from the file to the declarations tree. """

    warnings.warn(
        "merge_information is deprecated.\n" +
        "Please have a look at the changelog for an explanation (since 1.8.0)",
        DeprecationWarning)

    ext = os.path.splitext(fname)[1]

    if '.dll' == ext:
        parser = dll_file_parser_t(global_ns, fname)
    elif '.map' == ext:
        parser = map_file_parser_t(global_ns, fname)
    elif '.dylib' == ext or ('.so' == ext and platform.system() == 'Darwin'):
        parser = dylib_file_parser_t(global_ns, fname)
    elif '.so' == ext or '.so.' in os.path.basename(fname):
        parser = so_file_parser_t(global_ns, fname)
    else:
        raise RuntimeError(
            "Don't know how to read exported symbols from file '%s'" %
            fname)
    symbols = parser.parse()
    if runs_under_unittest:
        return symbols, parser
    else:
        return symbols



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        # TODO: once gccxml is removed; rename this to something like
        # annotate_tester
        self.header = "attributes_" + self.config.xml_generator + ".hpp"

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def test(self):

        numeric = self.global_ns.class_('numeric_t')
        do_nothing = numeric.mem_fun('do_nothing')
        arg = do_nothing.arguments[0]

        if "CastXML" in utils.xml_generator:
            if utils.xml_output_version >= 1.137:
                # This works since:
                # https://github.com/CastXML/CastXML/issues/25
                # https://github.com/CastXML/CastXML/pull/26
                # https://github.com/CastXML/CastXML/pull/27
                # The version bump to 1.137 came way later but this is the
                # only way to make sure the test is running correctly
                self.assertTrue("annotate(sealed)" == numeric.attributes)
                self.assertTrue("annotate(no throw)" == do_nothing.attributes)
                self.assertTrue("annotate(out)" == arg.attributes)
        else:
            self.assertTrue("gccxml(no throw)" == do_nothing.attributes)
            self.assertTrue("gccxml(out)" == arg.attributes)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2009 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "test_pattern_parser.hpp"
        self.config.cflags = "-std=c++11"

    def test_template_split_std_vector(self):
        """
        Demonstrate error in pattern parser, see #60

        """

        if self.config.xml_generator == "gccxml":
            return

        decls = parser.parse([self.header], self.config)

        for decl in declarations.make_flatten(decls):
            if "myClass" in decl.name:
                _ = decl.partial_name

    def test_matcher(self):
        """
        Run the matcher on all the templated classes.

        This exercises the whole pipeline even more.

        """

        if self.config.xml_generator == "gccxml":
            return

        decls = parser.parse([self.header], self.config)
        global_ns = declarations.get_global_namespace(decls)
        criteria = declarations.declaration_matcher(name="myClass")
        _ = declarations.matcher.find(criteria, global_ns)

    def test_split(self):
        """
        Test a bunch of tricky name/args splits. More combinations could be
        tested but this is already covering most of the cases.

        In test_template_split_std_vector we test for a specific case that
        was failing (in a real world scenario).
        Here we test more possible combinations to make sure the splitting
        method is robust enough.

        """

        p1 = "std::vector<char, std::allocator<char> >"
        p2 = "std::vector<int, std::allocator<int> >"
        args_list = [
            "const std::basic_string<char> &", "const int &", "const double &"]

        for arg in args_list:

            l = [p1]
            name, args = declarations.templates.split(
                "myClass0a<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass0a")
            self.assertEqual(args, l)

            l = [p1, p2]
            name, args = declarations.templates.split(
                "myClass0b<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass0b")
            self.assertEqual(args, l)

            l = [p1, p2, p2]
            name, args = declarations.templates.split(
                "myClass0c<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass0c")
            self.assertEqual(args, l)

            l = [p1 + " (" + arg + ")"]
            name, args = declarations.templates.split(
                "myClass1<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass1")
            self.assertEqual(args, l)

            l = [p1 + " (" + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass2<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass2")
            self.assertEqual(args, l)

            l = [p2 + " (" + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass3<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass3")
            self.assertEqual(args, l)

            l = [p1 + " (" + arg + ", " + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass4<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass4")
            self.assertEqual(args, l)

            l = [
                p1 + " (" + arg + ", " + arg + ", " + arg + ")",
                p1]
            name, args = declarations.templates.split(
                "myClass5<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass5")
            self.assertEqual(args, l)

            l = [
                p1,
                p1 + " (" + arg + ", " + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass6<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass6")
            self.assertEqual(args, l)

            l = [
                p2 + " (" + arg + ")",
                p1,
                p1 + " (" + arg + ", " + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass7<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass7")
            self.assertEqual(args, l)

            l = [
                p1,
                p2 + " (" + arg + ")",
                p1 + " (" + arg + ", " + arg + ", " + arg + ")"]
            name, args = declarations.templates.split(
                "myClass8<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass8")
            self.assertEqual(args, l)

            l = [
                p2 + " (" + arg + ")",
                p1 + " (" + arg + ", " + arg + ")",
                p1]
            name, args = declarations.templates.split(
                "myClass9<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass9")
            self.assertEqual(args, l)

            l = [
                p2 + " (" + arg + ")",
                p1 + " (" + arg + ", " + arg + ", " + arg + ")",
                p1,
                p2]
            name, args = declarations.templates.split(
                "myClass10<" + ", ".join(l) + ">")
            self.assertEqual(name, "myClass10")
            self.assertEqual(args, l)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def test_extract(self):
        data = [
            ('thiscall',
             '(public: __thiscall std::auto_ptr<class pof::number_t>' +
             '::auto_ptr<class pof::number_t>(class std::auto_ptr' +
             '<class pof::number_t> &))'),
            ('',  "(const pof::number_t::`vftable')")]

        for expected, text in data:
            got = declarations.CALLING_CONVENTION_TYPES.extract(text)
            self.assertTrue(
                got == expected, "Expected calling convention: %s, got %s" %
                (expected, got))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'const_volatile_arg.hpp'
        self.global_ns = None

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()
        self.global_ns = Test.global_ns

    def test(self):
        f = self.global_ns.free_fun('pygccxml_bug')
        t = f.arguments[0].decl_type
        self.assertTrue(isinstance(t, declarations.pointer_t))
        self.assertTrue(isinstance(t.base, declarations.volatile_t))
        self.assertTrue(isinstance(t.base.base, declarations.const_t))
        self.assertTrue(declarations.is_integral(t.base.base.base))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import sys
import unittest
import autoconfig
import subprocess
import parser_test_case
from pygccxml import parser


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = os.path.join(autoconfig.data_directory, 'core_cache.hpp')
        self.cache_file = os.path.join(
            autoconfig.data_directory,
            'pygccxml.cache')
        if os.path.exists(self.cache_file) and os.path.isfile(self.cache_file):
            os.remove(self.cache_file)

    def touch(self):
        # Need to change file.
        with open(self.header, "a+") as header:
            header.write("//touch")

    def test_update(self):

        # Save the content of the header file for later
        with open(self.header, "r") as old_header:
            content = old_header.read()

        declarations = parser.parse([self.header], self.config)
        cache = parser.file_cache_t(self.cache_file)
        cache.update(
            source_file=self.header,
            configuration=self.config,
            declarations=declarations,
            included_files=[])
        self.assertTrue(
            declarations == cache.cached_value(
                self.header,
                self.config),
            "cached declarations and source declarations are different")
        self.touch()
        self.assertTrue(
            cache.cached_value(self.header, self.config) is None,
            "cache didn't recognize that some files on disk has been changed")

        # We wrote a //touch in the header file. Just replace the file with the
        # original content. The touched file would be sometimes commited by
        # error as it was modified.
        with open(self.header, "w") as new_header:
            new_header.write(content)

    def test_from_file(self):
        declarations = parser.parse([self.header], self.config)
        cache = parser.file_cache_t(self.cache_file)
        cache.update(
            source_file=self.header,
            configuration=self.config,
            declarations=declarations,
            included_files=[])
        self.assertTrue(
            declarations == cache.cached_value(
                self.header,
                self.config),
            "cached declarations and source declarations are different")
        cache.flush()
        cache = parser.file_cache_t(self.cache_file)
        self.assertTrue(
            declarations == cache.cached_value(
                self.header,
                self.config),
            ("cached declarations and source declarations are different, " +
                "after pickling"))

    @staticmethod
    def test_reopen_cache():
        """
        Test opening cache files in a subprocess (with a clean environment).

        """

        env = os.environ.copy()

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))
        # Set the COVERAGE_PROCESS_START env. variable.
        # Allows to cover files run in a subprocess
        # http://nedbatchelder.com/code/coverage/subprocess.html
        env["COVERAGE_PROCESS_START"] = path + "/../.coveragerc"

        p = subprocess.Popen(
            [sys.executable, "unittests/reopen_cache_tester.py"],
            stdout=subprocess.PIPE,
            env=env)
        print(p.stdout.read())
        p.stdout.close()


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import autoconfig
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    declarations = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'type_traits.hpp'
        self.declarations = None

    def setUp(self):
        if not Test.declarations:
            Test.declarations = parser.parse([self.header], self.config)
        self.declarations = Test.declarations

    def __test_type_category(self, ns_name, controller):
        ns_control = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.namespace_t,
            name=ns_name)
        self.assertTrue(ns_control, "unable to find '%s' namespace" % ns_name)
        ns_yes = declarations.find_declaration(
            ns_control,
            decl_type=declarations.namespace_t,
            name='yes')
        self.assertTrue(ns_yes, "unable to find 'yes' namespace")
        ns_no = declarations.find_declaration(
            ns_control,
            decl_type=declarations.namespace_t,
            name='no')
        self.assertTrue(ns_no, "unable to find 'no' namespace")
        er = 'for type "%s" the answer to the question "%s" should be True'
        for decl in ns_yes.declarations:
            if isinstance(decl, declarations.variable_t):
                self.assertTrue(
                    controller(decl.decl_type),
                    er % (decl.decl_type.decl_string, ns_name))
            elif isinstance(decl, declarations.calldef_t) and \
                    decl.name.startswith('test_'):
                continue
            else:
                self.assertTrue(
                    controller(decl), er % (decl.decl_string, ns_name))
        er = 'for type "%s" the answer to the question "%s" should be False'
        for decl in ns_no.declarations:
            if isinstance(decl, declarations.calldef_t) and \
                    decl.name.startswith('test_'):
                continue

            if ("CastXML" in utils.xml_generator and
                    utils.xml_output_version < 1.138 and
                    decl.name in ['const_item', 'const_container']):
                # Skip this test to workaround CastXML bug.
                # See https://github.com/CastXML/CastXML/issues/55
                continue

            self.assertFalse(
                controller(decl),
                er % (decl.decl_string, ns_name))

    def __test_type_transformation(self, ns_name, transformer):
        ns_control = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.namespace_t,
            name=ns_name)
        self.assertTrue(ns_control, "unable to find '%s' namespace" % ns_name)
        ns_before = declarations.find_declaration(
            ns_control,
            decl_type=declarations.namespace_t,
            name='before')
        self.assertTrue(ns_before, "unable to find 'before' namespace")
        ns_after = declarations.find_declaration(
            ns_control,
            decl_type=declarations.namespace_t,
            name='after')
        self.assertTrue(ns_after, "unable to find 'after' namespace")

        for tbefore in ns_before.declarations:
            tafter = declarations.find_declaration(
                ns_after,
                name=tbefore.name)
            self.assertTrue(
                tafter,
                "unable to find transformed type definition for type '%s'" %
                tbefore.decl_string)
            transformed = transformer(tbefore)
            self.assertTrue(
                declarations.is_same(
                    transformed,
                    tafter),
                ("there is a difference between expected type '{0}' " +
                 "and result '{1}'. typedef name: {2}").format(
                    declarations.remove_declarated(tafter).decl_string,
                    declarations.remove_declarated(transformed).decl_string,
                    tbefore.decl_string))

    def test_is_enum(self):
        self.__test_type_category('is_enum', declarations.is_enum)

    def test_is_void(self):
        self.__test_type_category('is_void', declarations.is_void)

    def test_is_integral(self):
        self.__test_type_category('is_integral', declarations.is_integral)

    def test_is_pointer(self):
        self.__test_type_category('is_pointer', declarations.is_pointer)

    def test_is_const(self):
        self.__test_type_category('is_const', declarations.is_const)

    def test_is_volatile(self):
        self.__test_type_category('is_volatile', declarations.is_volatile)

    def test_is_reference(self):
        self.__test_type_category('is_reference', declarations.is_reference)

    def test_is_floating_point(self):
        self.__test_type_category(
            'is_floating_point',
            declarations.is_floating_point)

    def test_is_array(self):
        self.__test_type_category('is_array', declarations.is_array)

    def test_is_fundamental(self):
        self.__test_type_category(
            'is_fundamental',
            declarations.is_fundamental)

    def test_is_noncopyable(self):
        self.__test_type_category(
            'is_noncopyable',
            declarations.is_noncopyable)

    def test_is_std_ostream(self):
        self.__test_type_category(
            'is_std_ostream',
            declarations.is_std_ostream)

    def test_is_std_wostream(self):
        self.__test_type_category(
            'is_std_wostream',
            declarations.is_std_wostream)

    def test_is_calldef_pointer(self):
        self.__test_type_category(
            'is_calldef_pointer',
            declarations.is_calldef_pointer)

    def test_has_trivial_constructor(self):
        self.__test_type_category(
            'has_trivial_constructor',
            declarations.has_trivial_constructor)

    def test_has_public_constructor(self):
        self.__test_type_category(
            'has_public_constructor',
            declarations.has_public_constructor)

    def test_has_public_destructor(self):
        self.__test_type_category(
            'has_public_destructor',
            declarations.has_public_destructor)

    def test_has_any_non_copyconstructor(self):
        self.__test_type_category(
            'has_any_non_copyconstructor',
            declarations.has_any_non_copyconstructor)

    def test_has_copy_constructor(self):
        self.__test_type_category(
            'has_copy_constructor',
            declarations.has_copy_constructor)

    def test_is_base_and_derived(self):
        ns = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.namespace_t,
            name='is_base_and_derived')
        self.assertTrue(ns, "unable to find 'is_base_and_derived' namespace")
        base = declarations.find_declaration(
            ns.declarations,
            decl_type=declarations.class_t,
            name='base')
        derived = declarations.find_declaration(
            ns.declarations,
            decl_type=declarations.class_t,
            name='derived')
        self.assertTrue(
            base and derived and declarations.is_base_and_derived(
                base,
                derived))
        self.assertTrue(
            base and derived and
            declarations.is_base_and_derived(base, (derived, derived)))

        unrelated1 = declarations.find_declaration(
            ns.declarations,
            decl_type=declarations.class_t,
            name='unrelated1')

        unrelated2 = declarations.find_declaration(
            ns.declarations,
            decl_type=declarations.class_t,
            name='unrelated2')
        self.assertTrue(
            base and derived and not declarations.is_base_and_derived(
                unrelated1,
                unrelated2))

    def test_is_same(self):
        self.assertTrue(
            declarations.is_same(
                declarations.int_t,
                declarations.int_t))
        self.assertFalse(
            declarations.is_same(
                declarations.int_t,
                declarations.float_t))

    def test_remove_const(self):
        self.__test_type_transformation(
            'remove_const',
            declarations.remove_const)

    def test_remove_reference(self):
        self.__test_type_transformation(
            'remove_reference',
            declarations.remove_reference)

    def test_remove_volatile(self):
        self.__test_type_transformation(
            'remove_volatile',
            declarations.remove_volatile)

    def test_remove_cv(self):
        self.__test_type_transformation('remove_cv', declarations.remove_cv)

    def test_remove_pointer(self):
        self.__test_type_transformation(
            'remove_pointer',
            declarations.remove_pointer)

    def test_is_unary_binary_operator(self):
        operator_not = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::dummy::operator!')
        self.assertTrue(operator_not, 'operator! was not found')
        self.assertTrue(
            declarations.is_unary_operator(operator_not),
            'operator! should be idenitified as unary operator')
        self.assertTrue(
            not declarations.is_binary_operator(operator_not),
            'operator! should be idenitified as unary operator')

        operator_class_p = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::dummy::operator+')
        self.assertTrue(operator_class_p, 'operator+ was not found')
        self.assertTrue(
            not declarations.is_unary_operator(operator_class_p),
            'operator+ should be idenitified as binary operator')
        self.assertTrue(
            declarations.is_binary_operator(operator_class_p),
            'operator! should be idenitified as binary operator')

        operator_class_pp = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::dummy::operator++')
        self.assertTrue(operator_class_pp, 'operator++ was not found')
        self.assertTrue(
            declarations.is_unary_operator(operator_class_pp),
            'operator++ should be idenitified as unary operator')
        self.assertTrue(
            not declarations.is_binary_operator(operator_class_pp),
            'operator++ should be idenitified as unary operator')

        operator_pp = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::operator++')
        self.assertTrue(operator_pp, 'operator++ was not found')
        self.assertTrue(
            declarations.is_unary_operator(operator_pp),
            'operator++ should be idenitified as unary operator')
        self.assertTrue(
            not declarations.is_binary_operator(operator_pp),
            'operator++ should be idenitified as unary operator')

        operator_mm = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::operator*')
        self.assertTrue(operator_mm, 'operator-- was not found')
        self.assertTrue(
            not declarations.is_unary_operator(operator_mm),
            'operator-- should be idenitified as binary operator')
        self.assertTrue(
            declarations.is_binary_operator(operator_mm),
            'operator-- should be idenitified as binray operator')

        operator_pe = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.operator_t,
            fullname='::is_unary_operator::operator+=')
        self.assertTrue(operator_pe, 'operator+= was not found')
        self.assertTrue(
            not declarations.is_unary_operator(operator_pe),
            'operator+= should be idenitified as binary operator')
        self.assertTrue(
            declarations.is_binary_operator(operator_pe),
            'operator+= should be idenitified as binray operator')

    def __is_convertible_impl(self, decl):
        defs = decl.bases[0].related_class.declarations
        source_type = declarations.find_declaration(defs, name='source_type')
        target_type = declarations.find_declaration(defs, name='target_type')
        expected_type = declarations.find_declaration(
            defs,
            name='expected',
            decl_type=declarations.enumeration_t)
        expected_value = bool(expected_type.get_name2value_dict()['value'])
        self.assertTrue(
            expected_value == declarations.is_convertible(
                source_type,
                target_type),
            'Check conversion failed for ' +
            decl.name)

    def test_is_convertible(self):
        ns_is_convertible = declarations.find_declaration(
            self.declarations,
            decl_type=declarations.namespace_t,
            name="is_convertible")

        self.assertTrue(
            ns_is_convertible,
            "namespace is_convertible was not found")
        for tester in [
            decl for decl in ns_is_convertible.declarations if
                decl.name.startswith('x')]:

            self.__is_convertible_impl(tester)


class missing_decls_tester_t(unittest.TestCase):

    def test(self):
        config = autoconfig.cxx_parsers_cfg.gccxml
        code = "struct const_item{ const int values[10]; };"
        global_ns = parser.parse_string(code, config)[0]
        ci = global_ns.class_('const_item')
        if ("CastXML" not in utils.xml_generator or
                utils.xml_output_version >= 1.138):
            # Prior to version 1.138, CastXML would incorrect create a default
            # constructor definition.
            # See https://github.com/CastXML/CastXML/issues/55
            # Copy constructor, destructor, variable
            self.assertEqual(len(ci.declarations), 3)

# class tester_diff_t( parser_test_case.parser_test_case_t ):
    # COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    # declarations = None
    # def __init__(self, *args ):
        # parser_test_case.parser_test_case_t.__init__( self, *args )
        # self.header = 'type_traits.hpp'
        # self.declarations = None

    # def setUp(self):
        # if not Test.declarations:
        # Test.declarations = parser.parse([self.header], self.config)
        # self.declarations = Test.declarations

    # def test( self ):
        # x = declarations.find_declaration( self.declarations
        # , decl_type=declarations.typedef_t
        # , name="s2s_multimap_type" )
        # print declarations.is_noncopyable( x)
        # declarations.print_declarations(
        #    [declarations.class_traits.get_declaration( x )] )


class class_traits_tester_t(unittest.TestCase):

    def test_get_declaration(self):
        code = """
            namespace A{
            struct B{
                int c;
            };

            template <class T>
            struct C: public T{
                int d;
            };

            template <class T>
            struct D{
                int dD;
            };

            typedef C<B> easy;
            typedef D<easy> Deasy;

            inline void instantiate(){
                int val = sizeof(easy);
            }

            }
        """

        global_ns = parser.parse_string(
            code,
            autoconfig.cxx_parsers_cfg.gccxml)
        global_ns = declarations.get_global_namespace(global_ns)
        easy = global_ns.typedef('easy')
        declarations.class_traits.get_declaration(easy)
        deasy = global_ns.typedef('Deasy')
        d_a = declarations.class_traits.get_declaration(deasy)
        self.assertTrue(isinstance(d_a, declarations.class_types))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(class_traits_tester_t))
    suite.addTest(unittest.makeSuite(Test))
    suite.addTest(unittest.makeSuite(missing_decls_tester_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "test_copy_constructor.hpp"

    def setUp(self):
        decls = parser.parse([self.header], self.config)
        self.global_ns = declarations.get_global_namespace(decls)

    def test(self):
        """
        Check the is_copy_constructor method.

        This fails when using CastXML, see issue #27.

        """

        tclass = self.global_ns.class_("test")
        ctors = []
        for decl in tclass.declarations:
            if isinstance(decl, declarations.constructor_t):
                ctors.append(decl)

        # test::test(test const & t0) [copy constructor]
        self.assertTrue(declarations.is_copy_constructor(ctors[0]))
        # test::test(float const & t0) [constructor]
        self.assertFalse(declarations.is_copy_constructor(ctors[1]))
        # test::test(myvar t0) [constructor]
        self.assertFalse(declarations.is_copy_constructor(ctors[2]))

        t2class = self.global_ns.class_("test2")
        ctors = []
        for decl in t2class.declarations:
            if isinstance(decl, declarations.constructor_t):
                ctors.append(decl)

        # GCCXML and CastXML return the constructors in a different order.
        # I hope this index inversion will cover the two cases. If different
        # compilers give other orders, we will need to find a nicer solution.
        if "CastXML" in utils.xml_generator:
            positions = [0, 1]
        elif "GCC" in utils.xml_generator:
            positions = [1, 0]

        # test2::test2() [constructor]
        self.assertFalse(declarations.is_copy_constructor(ctors[positions[0]]))
        # test2::test2(test2 const & arg0) [copy constructor]
        self.assertTrue(declarations.is_copy_constructor(ctors[positions[1]]))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__files = [
            'core_ns_join_1.hpp',
            'core_ns_join_2.hpp',
            'core_ns_join_3.hpp',
            'core_membership.hpp',
            'core_class_hierarchy.hpp',
            'core_types.hpp',
            'core_diamand_hierarchy_base.hpp',
            'core_diamand_hierarchy_derived1.hpp',
            'core_diamand_hierarchy_derived2.hpp',
            'core_diamand_hierarchy_final_derived.hpp',
            'core_overloads_1.hpp',
            'core_overloads_2.hpp']

    def test(self):
        prj_reader = parser.project_reader_t(self.config)
        decls = prj_reader.read_files(
            self.__files,
            compilation_mode=parser.COMPILATION_MODE.ALL_AT_ONCE)
        files = declarations.declaration_files(decls)
        result = set()
        for fn in files:
            result.add(os.path.split(fn)[1])
        self.assertTrue(set(self.__files).issubset(result))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import parser_test_case
from pygccxml import parser


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.content = "abra cadabra " + os.linesep

    def test_gccxml_on_input_with_errors(self):
        self.assertRaises(
            RuntimeError,
            parser.parse_string,
            self.content,
            self.config)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import sys
import unittest

import decl_string_tester
import declaration_files_tester
import declarations_comparison_tester
import declarations_tester
import file_cache_tester
import gccxml_runner_tester
import project_reader_correctness_tester
import source_reader_tester
import start_with_declarations_tester
import templates_tester
import type_traits_tester
import core_tester
import xmlfile_reader_tester
import text_reader_tester
import hierarchy_traveling
import patcher_tester
import call_invocation_tester
import bit_fields_tester
import complex_types_tester
import cached_source_file_tester
import variable_matcher_tester
import namespace_matcher_tester
import calldef_matcher_tester
import filters_tester
import cache_enums_tester
import decl_printer_tester
import typedefs_tester
import non_copyable_classes_tester
# import demangled_tester
import unnamed_enums_bug_tester
import vector_traits_tester
import string_traits_tester
import declarations_cache_tester
import has_binary_operator_traits_tester
import algorithms_cache_tester
import dependencies_tester
import free_operators_tester
import remove_template_defaults_tester
import find_container_traits_tester
import attributes_tester
import type_as_exception_bug_tester
import copy_constructor_tester
import plain_c_tester
import function_traits_tester
import better_templates_matcher_tester
import declaration_matcher_tester
# import undname_creator_tester
import calling_convention_tester
import const_volatile_arg_tester
import array_bug_tester
import gccxml10183_tester
import gccxml10184_tester
import gccxml10185_tester
import inline_specifier_tester
import test_create_decl_string
import pep8_tester
import example_tester
import test_utils
import test_va_list_tag_removal
import test_copy_constructor
import test_cpp_standards
import unnamed_classes_tester
import test_map_gcc5
import test_argument_without_name
import test_smart_pointer
import test_pattern_parser
import test_function_pointer

testers = [
    # , demangled_tester # failing right now
    # , undname_creator_tester # failing right now
    pep8_tester,
    decl_string_tester,
    declaration_files_tester,
    declarations_comparison_tester,
    declarations_tester, file_cache_tester,
    gccxml_runner_tester,
    project_reader_correctness_tester,
    source_reader_tester,
    start_with_declarations_tester,
    templates_tester,
    type_traits_tester,
    core_tester,
    xmlfile_reader_tester,
    text_reader_tester,
    hierarchy_traveling,
    patcher_tester,
    call_invocation_tester,
    bit_fields_tester,
    complex_types_tester,
    cached_source_file_tester,
    variable_matcher_tester,
    namespace_matcher_tester,
    calldef_matcher_tester,
    filters_tester,
    cache_enums_tester,
    decl_printer_tester,
    typedefs_tester,
    non_copyable_classes_tester,
    unnamed_enums_bug_tester,
    vector_traits_tester,
    string_traits_tester,
    declarations_cache_tester,
    has_binary_operator_traits_tester,
    algorithms_cache_tester,
    dependencies_tester,
    free_operators_tester,
    remove_template_defaults_tester,
    find_container_traits_tester,
    attributes_tester,
    type_as_exception_bug_tester,
    plain_c_tester,
    function_traits_tester,
    better_templates_matcher_tester,
    declaration_matcher_tester,
    calling_convention_tester,
    const_volatile_arg_tester,
    array_bug_tester,
    gccxml10183_tester,
    gccxml10184_tester,
    gccxml10185_tester,
    inline_specifier_tester,
    test_create_decl_string,
    example_tester,
    test_utils,
    test_va_list_tag_removal,
    test_copy_constructor,
    test_cpp_standards,
    unnamed_classes_tester,
    test_map_gcc5,
    test_argument_without_name,
    test_smart_pointer,
    test_pattern_parser,
    test_function_pointer
]

if 'posix' in os.name:
    testers.append(copy_constructor_tester)

if os.path.isfile("test_cost.log"):
    # Remove the cost log file when tests are run again.
    # See the parser_test_case which generates this file.
    os.remove("test_cost.log")


def create_suite():
    main_suite = unittest.TestSuite()
    for tester in testers:
        main_suite.addTest(tester.create_suite())
    return main_suite


def run_suite():
    result = unittest.TextTestRunner(verbosity=2).run(create_suite())
    error_desc = 'EXCEPTION IN SAFE SELECT 9'
    all_errors = result.failures + result.errors
    for test_case, description in all_errors:
        if error_desc not in description:
            return 1
    return 0

if __name__ == "__main__":
    sys.exit(run_suite())



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""
Defines C++ parser configuration classes.

"""

import os
import copy
import platform
import subprocess
import warnings
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser as ConfigParser
from .. import utils


class parser_configuration_t(object):

    """
    C++ parser configuration holder

    This class serves as a base class for the parameters that can be used
    to customize the call to a C++ parser.

    This class also allows users to work with relative files paths. In this
    case files are searched in the following order:

       1. current directory
       2. working directory
       3. additional include paths specified by the user

    """

    def __init__(
            self,
            working_directory='.',
            include_paths=None,
            define_symbols=None,
            undefine_symbols=None,
            cflags="",
            compiler=None,
            xml_generator="castxml",
            keep_xml=False,
            compiler_path=None,
            flags=None):

        object.__init__(self)
        self.__working_directory = working_directory

        if not include_paths:
            include_paths = []
        self.__include_paths = include_paths

        if not define_symbols:
            define_symbols = []
        self.__define_symbols = define_symbols

        if not undefine_symbols:
            undefine_symbols = []
        self.__undefine_symbols = undefine_symbols

        self.__cflags = cflags

        self.__compiler = compiler

        self.__xml_generator = xml_generator

        self.__keep_xml = keep_xml

        if flags is None:
            flags = []
        self.__flags = flags

        # If no compiler path was set and we are using castxml, set the path
        self.__compiler_path = create_compiler_path(
            xml_generator, compiler_path)

    def clone(self):
        raise NotImplementedError(self.__class__.__name__)

    @property
    def working_directory(self):
        return self.__working_directory

    @working_directory.setter
    def working_directory(self, working_dir):
        self.__working_directory = working_dir

    @property
    def include_paths(self):
        """list of include paths to look for header files"""
        return self.__include_paths

    @property
    def define_symbols(self):
        """list of "define" directives """
        return self.__define_symbols

    @property
    def undefine_symbols(self):
        """list of "undefine" directives """
        return self.__undefine_symbols

    @property
    def compiler(self):
        """get compiler name to simulate"""
        return self.__compiler

    @compiler.setter
    def compiler(self, compiler):
        """set compiler name to simulate"""
        self.__compiler = compiler

    @property
    def xml_generator(self):
        """get xml_generator (gccxml or castxml)"""
        return self.__xml_generator

    @xml_generator.setter
    def xml_generator(self, xml_generator):
        """set xml_generator (gccxml or castxml)"""
        if "real" in xml_generator:
            # Support for gccxml.real from newer gccxml package
            # Can be removed once gccxml support is dropped.
            xml_generator = "gccxml"
        self.__xml_generator = xml_generator

    @property
    def keep_xml(self):
        """Are xml files kept after errors."""
        return self.__keep_xml

    @keep_xml.setter
    def keep_xml(self, keep_xml):
        """Set if xml files kept after errors."""
        self.__keep_xml = keep_xml

    @property
    def flags(self):
        """Optional flags for pygccxml."""
        return self.__flags

    @flags.setter
    def flags(self, flags):
        """Optional flags for pygccxml."""
        if flags is None:
            flags = []
        self.__flags = flags

    @property
    def compiler_path(self):
        """Get the path for the compiler."""
        return self.__compiler_path

    @compiler_path.setter
    def compiler_path(self, compiler_path):
        """Set the path for the compiler."""
        self.__compiler_path = compiler_path

    @property
    def cflags(self):
        """additional flags to pass to compiler"""
        return self.__cflags

    @cflags.setter
    def cflags(self, val):
        self.__cflags = val

    def append_cflags(self, val):
        self.__cflags = self.__cflags + ' ' + val

    def __ensure_dir_exists(self, dir_path, meaning):
        if os.path.isdir(dir_path):
            return
        if os.path.exists(self.working_directory):
            raise RuntimeError(
                '%s("%s") does not exist!' % (meaning, dir_path))
        else:
            raise RuntimeError(
                '%s("%s") should be "directory", not a file.' %
                (meaning, dir_path))

    def raise_on_wrong_settings(self):
        """validates the configuration settings and raises RuntimeError on
        error"""
        self.__ensure_dir_exists(self.working_directory, 'working directory')
        for idir in self.include_paths:
            self.__ensure_dir_exists(idir, 'include directory')


class xml_generator_configuration_t(parser_configuration_t):
    """
    Configuration object to collect parameters for invoking gccxml or castxml.

    This class serves as a container for the parameters that can be used
    to customize the call to gccxml or castxml.

    """

    def __init__(
            self,
            gccxml_path='',
            xml_generator_path='',
            working_directory='.',
            include_paths=None,
            define_symbols=None,
            undefine_symbols=None,
            start_with_declarations=None,
            ignore_gccxml_output=False,
            cflags="",
            compiler=None,
            xml_generator="castxml",
            keep_xml=False,
            compiler_path=None,
            flags=None):

        parser_configuration_t.__init__(
            self,
            working_directory=working_directory,
            include_paths=include_paths,
            define_symbols=define_symbols,
            undefine_symbols=undefine_symbols,
            cflags=cflags,
            compiler=compiler,
            xml_generator=xml_generator,
            keep_xml=keep_xml,
            compiler_path=compiler_path,
            flags=flags)

        if gccxml_path != '':
            self.__gccxml_path = gccxml_path
        self.__xml_generator_path = xml_generator_path

        if not start_with_declarations:
            start_with_declarations = []
        self.__start_with_declarations = start_with_declarations

        self.__ignore_gccxml_output = ignore_gccxml_output

    def clone(self):
        return copy.deepcopy(self)

    @property
    def xml_generator_path(self):
        """
        XML generator binary location

        """

        return self.__xml_generator_path

    @xml_generator_path.setter
    def xml_generator_path(self, new_path):
        self.__xml_generator_path = new_path

    @property
    def start_with_declarations(self):
        """list of declarations gccxml should start with, when it dumps
        declaration tree"""
        return self.__start_with_declarations

    @property
    def ignore_gccxml_output(self):
        """set this property to True, if you want pygccxml to ignore any
            error warning that comes from gccxml"""
        return self.__ignore_gccxml_output

    @ignore_gccxml_output.setter
    def ignore_gccxml_output(self, val=True):
        self.__ignore_gccxml_output = val

    def raise_on_wrong_settings(self):
        super(xml_generator_configuration_t, self).raise_on_wrong_settings()
        if os.path.isfile(self.xml_generator_path):
            return
        if os.name == 'nt':
            gccxml_name = 'gccxml' + '.exe'
            environment_var_delimiter = ';'
        elif os.name == 'posix':
            gccxml_name = 'gccxml'
            environment_var_delimiter = ':'
        else:
            raise RuntimeError(
                'unable to find out location of the xml generator')
        may_be_gccxml = os.path.join(self.xml_generator_path, gccxml_name)
        if os.path.isfile(may_be_gccxml):
            self.xml_generator_path = may_be_gccxml
        else:
            for path in os.environ['PATH'].split(environment_var_delimiter):
                xml_generator_path = os.path.join(path, gccxml_name)
                if os.path.isfile(xml_generator_path):
                    self.xml_generator_path = xml_generator_path
                    break
            else:
                msg = (
                    'xml_generator_path("%s") should exists or to be a ' +
                    'valid file name.') % self.xml_generator_path
                raise RuntimeError(msg)


class _StringDeprecationWrapper(str):
    """
    A small wrapper class useful when deprecation strings.

    This class is not part of the public API.

    """

    def __new__(cls, content):
        cls.content = content
        return str.__new__(cls, content)

    def __str__(self):
        warnings.warn(
            "gccxml_configuration_example is deprecated. There is an " +
            "example file here if you need one: unittests/xml_generator.cfg"
            "This will be removed in version 1.9.0",
            DeprecationWarning)
        return self.content


gccxml_configuration_example = _StringDeprecationWrapper(
    """
[gccxml]
#path to gccxml executable file - optional, if not provided, os.environ['PATH']
#variable is used to find it
gccxml_path=(deprecated)
xml_generator_path=
#gccxml working directory - optional, could be set to your source code
directory
working_directory=
#additional include directories, separated by ';'
include_paths=
#gccxml has a nice algorithms, which selects what C++ compiler to emulate.
#You can explicitly set what compiler it should emulate.
#Valid options are: g++, msvc6, msvc7, msvc71, msvc8, cl.
compiler=
# gccxml or castxml
xml_generator=
# Do we keep xml files or not after errors
keep_xml=
# Set the path to the compiler
compiler_path=
""")


def load_xml_generator_configuration(configuration, **defaults):
    """
    Loads CastXML or GCC-XML configuration.

    Args:
         configuration (string|configparser.ConfigParser): can be
             a string (file path to a configuration file) or
             instance of :class:`configparser.ConfigParser`.
         defaults: can be used to override single configuration values.

    Returns:
        :class:`.xml_generator_configuration_t`: a configuration object


    The file passed needs to be in a format that can be parsed by
    :class:`configparser.ConfigParser`.

    An example configuration file skeleton can be found
    `here <https://github.com/gccxml/pygccxml/blob/develop/
    unittests/xml_generator.cfg>`_.

    """
    parser = configuration
    if utils.is_str(configuration):
        parser = ConfigParser()
        parser.read(configuration)

    # Create a new empty configuration
    cfg = xml_generator_configuration_t()

    values = defaults
    if not values:
        values = {}

    if parser.has_section('gccxml'):
        warnings.warn(
            "The [gccxml] section in your configuration file is deprecated. "
            "Please use [xml_generator] instead. This will no more work with "
            "version 1.9.0")
        for name, value in parser.items('gccxml'):
            if value.strip():
                values[name] = value

    if parser.has_section('xml_generator'):
        for name, value in parser.items('xml_generator'):
            if value.strip():
                values[name] = value

    for name, value in values.items():
        if isinstance(value, str):
            value = value.strip()
        if name == 'gccxml_path':
            cfg.gccxml_path = value
        if name == 'xml_generator_path':
            cfg.xml_generator_path = value
        elif name == 'working_directory':
            cfg.working_directory = value
        elif name == 'include_paths':
            for p in value.split(';'):
                p = p.strip()
                if p:
                    cfg.include_paths.append(p)
        elif name == 'compiler':
            cfg.compiler = value
        elif name == 'xml_generator':
            cfg.xml_generator = value
        elif name == 'keep_xml':
            cfg.keep_xml = value
        elif name == 'cflags':
            cfg.cflags = value
        elif name == 'flags':
            cfg.flags = value
        elif name == 'compiler_path':
            cfg.compiler_path = value
        else:
            print('\n%s entry was ignored' % name)

    # If no compiler path was set and we are using castxml, set the path
    # Here we overwrite the default configuration done in the cfg because
    # the xml_generator was set through the setter after the creation of a new
    # emppty configuration object.
    cfg.compiler_path = create_compiler_path(
        cfg.xml_generator, cfg.compiler_path)

    return cfg


def create_compiler_path(xml_generator, compiler_path):
    """
    Try to guess a path for the compiler.

    If you want ot use a specific compiler, please provide the compiler
    path manually, as the guess may not be what you are expecting.
    Providing the path can be done by passing it as an argument (compiler_path)
    to the xml_generator_configuration_t() or by defining it in your pygccxml
    configuration file.

    """

    if xml_generator == 'castxml' and compiler_path is None:
        if platform.system() == 'Windows':
            # Look for msvc
            p = subprocess.Popen(
                ['where', 'cl'],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            compiler_path = p.stdout.read().decode("utf-8").rstrip()
            p.stdout.close()
            p.stderr.close()
            # No mscv found; look for mingw
            if compiler_path == '':
                p = subprocess.Popen(
                    ['where', 'mingw'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE)
                compiler_path = p.stdout.read().decode("utf-8").rstrip()
                p.stdout.close()
                p.stderr.close()
        else:
            # OS X or Linux
            # Look for clang first, then gcc
            p = subprocess.Popen(
                ['which', 'clang++'],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            compiler_path = p.stdout.read().decode("utf-8").rstrip()
            p.stdout.close()
            p.stderr.close()
            # No clang found; use gcc
            if compiler_path == '':
                p = subprocess.Popen(
                    ['which', 'c++'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE)
                compiler_path = p.stdout.read().decode("utf-8").rstrip()
                p.stdout.close()
                p.stderr.close()

        if compiler_path == "":
            compiler_path = None

    return compiler_path


gccxml_configuration_t = utils.utils.DeprecationWrapper(
    xml_generator_configuration_t,
    "gccxml_configuration_t",
    "xml_generator_configuration_t",
    "1.9.0")
load_gccxml_configuration = utils.utils.DeprecationWrapper(
    load_xml_generator_configuration,
    "load_gccxml_configuration",
    "load_xml_generator_configuration",
    "1.9.0")


if __name__ == '__main__':
    print(load_xml_generator_configuration('xml_generator.cfg').__dict__)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import bz2
import unittest
import autoconfig
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.global_ns = None
        self.xml_path = None

    def setUp(self):
        if not self.global_ns:

            # Extract the xml file from the bz2 archive
            bz2_path = os.path.join(
                autoconfig.data_directory,
                'ogre.1.7.xml.bz2')
            self.xml_path = os.path.join(
                autoconfig.data_directory,
                'ogre.1.7.xml')
            with open(self.xml_path, 'wb') as new_file:
                # bz2.BZ2File can not be used in a with statement in python 2.6
                bz2_file = bz2.BZ2File(bz2_path, 'rb')
                for data in iter(lambda: bz2_file.read(100 * 1024), b''):
                    new_file.write(data)
                bz2_file.close()

            reader = parser.source_reader_t(autoconfig.cxx_parsers_cfg.gccxml)
            self.global_ns = declarations.get_global_namespace(
                reader.read_xml_file(
                    self.xml_path))
            self.global_ns.init_optimizer()

    def tearDown(self):
        # Delete the extracted xml file
        os.remove(self.xml_path)

    def test(self):
        for x in self.global_ns.typedefs('SettingsMultiMap'):
            self.assertTrue(not declarations.is_noncopyable(x))

        for x in self.global_ns.typedefs('SettingsIterator'):
            self.assertTrue(not declarations.is_noncopyable(x))

        for x in self.global_ns.typedefs('SectionIterator'):
            self.assertTrue(not declarations.is_noncopyable(x))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'include_all.hpp'
        self.global_ns = None

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()
        self.global_ns = Test.global_ns

    def test_variable(self):
        ns_vars = self.global_ns.namespace('::declarations::variables')
        static_var = ns_vars.variable('static_var')
        dependencies = static_var.i_depend_on_them()
        self.assertTrue(len(dependencies) == 1)
        self.assertTrue(dependencies[0].declaration is static_var)
        self.assertTrue(dependencies[0].depend_on_it.decl_string == 'int')

        m_mutable = ns_vars.variable('m_mutable')
        dependencies = m_mutable.i_depend_on_them()
        self.assertTrue(len(dependencies) == 1)
        self.assertTrue(dependencies[0].declaration is m_mutable)
        self.assertTrue(dependencies[0].depend_on_it.decl_string == 'int')

    def test_class(self):
        ns_vars = self.global_ns.namespace('::declarations::variables')

        cls = ns_vars.class_('struct_variables_t')
        dependencies = cls.i_depend_on_them()
        if '0.9' in utils.xml_generator or 'CastXML' in utils.xml_generator:
            # GCCXML R122 adds compiler generated constructors/destructors
            # and operator= to the class, if it has
            dependencies = [
                d for d in dependencies if not d.declaration.is_artificial]
            self.assertTrue(len(dependencies) == 1)
        else:
            # compiler generated copy constructor
            self.assertTrue(len(dependencies) == 2)

        m_mutable = ns_vars.variable('m_mutable')
        dependencies = [
            dependency for dependency in dependencies if
            dependency.declaration is m_mutable]
        self.assertTrue(len(dependencies) == 1)
        self.assertTrue(dependencies[0].depend_on_it.decl_string == 'int')
        self.assertTrue(dependencies[0].access_type == 'public')

        ns_dh = self.global_ns.namespace('::core::diamand_hierarchy')
        fd_cls = ns_dh.class_('final_derived_t')
        derived1_cls = ns_dh.class_('derived1_t')
        dependencies = fd_cls.i_depend_on_them()
        dependencies = [
            dependency for dependency in dependencies if
            dependency.depend_on_it is derived1_cls]
        self.assertTrue(len(dependencies) == 1)
        self.assertTrue(dependencies[0].depend_on_it is derived1_cls)
        self.assertTrue(dependencies[0].access_type == 'public')

    def test_calldefs(self):
        ns = self.global_ns.namespace('::declarations::calldef')
        return_default_args = ns.calldef('return_default_args')
        dependencies = return_default_args.i_depend_on_them()
        self.assertTrue(len(dependencies) == 3)
        used_types = [
            dependency.depend_on_it.decl_string for dependency in dependencies]
        self.assertTrue(used_types == ['int', 'int', 'bool'])

        some_exception = ns.class_('some_exception_t')
        other_exception = ns.class_('other_exception_t')
        calldef_with_throw = ns.calldef('calldef_with_throw')
        dependencies = calldef_with_throw.i_depend_on_them()
        self.assertTrue(len(dependencies) == 3)
        dependencies = [
            dependency for dependency in dependencies if
            dependency.depend_on_it in (some_exception, other_exception)]
        self.assertTrue(len(dependencies) == 2)

    def test_coverage(self):
        self.global_ns.i_depend_on_them()


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import autoconfig
import parser_test_case

from pygccxml import utils
from pygccxml import parser
from pygccxml import declarations


class tester_impl_t(parser_test_case.parser_test_case_t):

    def __init__(self, architecture, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'demangled.hpp'
        self.global_ns = None
        self.architecture = architecture

    def setUp(self):
        reader = parser.source_reader_t(self.config)
        decls = None
        if 32 == self.architecture:
            decls = reader.read_file(self.header)
        else:
            original_get_architecture = utils.get_architecture
            utils.get_architecture = lambda: 64
            decls = reader.read_xml_file(
                os.path.join(
                    autoconfig.data_directory,
                    'demangled_tester_64bit.xml'))
            utils.get_architecture = original_get_architecture
        self.global_ns = declarations.get_global_namespace(decls)

    def test(self):
        demangled = self.global_ns.namespace('demangled')

        if "CastXML" in utils.xml_generator:
            # Do not test demangled name for CastXML
            return True

        if 32 == self.architecture:
            if '0.9' in demangled.compiler:
                if 0:  # platform.machine() == 'x86_64':
                    cls = demangled.class_(
                        'item_t<25214903917ul, 11ul, 2147483648ul>')
                    self.assertTrue(
                        cls._name == 'item_t<25214903917ul,11ul,2147483648ul>',
                        cls._name)
                else:
                    cls = demangled.class_(
                        'item_t<3740067437ul, 11ul, 2147483648ul>')
                    self.assertTrue(
                        cls._name == 'item_t<3740067437ul,11ul,2147483648ul>')
            else:
                cls = demangled.class_('item_t<3740067437l, 11l, 2147483648l>')
                self.assertTrue(
                    cls._name == 'item_t<0x0deece66d,11,0x080000000>')
        else:
            cls = demangled.class_("item_t<25214903917l, 11l, 2147483648l>")
            self.assertTrue(cls._name == 'item_t<25214903917,11,2147483648>')

    def test_free_function(self):
        f = self.global_ns.free_functions('set_a', allow_empty=True)
        if not f:
            return
        f = f[0]
        self.assertTrue(f.mangled)


class tester_32_t(tester_impl_t):

    def __init__(self, *args):
        tester_impl_t.__init__(self, 32, *args)


class tester_64_t(tester_impl_t):

    def __init__(self, *args):
        tester_impl_t.__init__(self, 64, *args)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(tester_32_t))
    suite.addTest(unittest.makeSuite(tester_64_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils

code = \
    """
template <typename> class A {};
template <typename T> void f(A<T> const& a){ A<__typeof__(a)>(); }
template void f<int>(A<int> const& a);
"""


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)

    def test(self):
        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))

        # TODO: demangled attribute does not existe for castxml
        # and will not be added. Remove this test once gccxml
        # support is dropped.
        if 'GCCXML' in utils.xml_generator:
            global_ns.decl('A<int>')
            f = global_ns.free_fun('f')
            self.assertTrue(f.demangled == 'void f<int>(A<int> const&)')


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import platform
import autoconfig
import parser_test_case

from pygccxml import utils
from pygccxml import parser


class tester_impl_t(parser_test_case.parser_test_case_t):

    def __init__(self, architecture, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.architecture = architecture
        self.global_ns = None
        self.__cxx_std = utils.cxx_standard(self.config.cflags)

    def test_enum_patcher(self):
        fix_enum = self.global_ns.free_fun("fix_enum")
        default_val = fix_enum.arguments[0].default_value
        if self.__cxx_std.is_cxx11_or_greater:
            val = "::ns1::ns2::fruit::apple"
        else:
            val = "::ns1::ns2::apple"
        self.assertEqual(default_val, val)

        if 32 == self.architecture or "CastXML" in utils.xml_generator:
            fix_enum2 = self.global_ns.free_fun("fix_enum2")
            default_val = fix_enum2.arguments[0].default_value
            self.assertEqual(default_val, val)

            ns1 = self.global_ns.namespace("ns1")
            ns2 = ns1.namespace("ns2")
            fix_enum2 = ns2.free_fun("fix_enum2")
            default_val = fix_enum2.arguments[0].default_value
            self.assertEqual(default_val, val)

            fix_enum3 = self.global_ns.free_fun("fix_enum3")
            default_val = fix_enum3.arguments[0].default_value
            val = val.replace("apple", "orange")
            self.assertEqual(default_val, val)

            if self.__cxx_std.is_cxx11_or_greater:
                fix_enum4 = self.global_ns.free_fun("fix_enum4")
                default_val = fix_enum4.arguments[0].default_value
                self.assertEqual(default_val, "::ns4::color::blue")

                fix_enum5 = self.global_ns.free_fun("fix_enum5")
                default_val = fix_enum5.arguments[0].default_value
                self.assertEqual(default_val, "::ns4::color::blue")

            lpe = self.global_ns.free_fun("log_priority_enabled")
            default_val = lpe.arguments[0].default_value
            if self.__cxx_std.is_cxx11_or_greater:
                val = "(long int)" + \
                    "(::ACE_Log_Priority_Index::LM_INVALID_BIT_INDEX)"
            else:
                val = "(long int)(::LM_INVALID_BIT_INDEX)"
            self.assertEqual(default_val, val)

    def test_numeric_patcher(self):
        fix_numeric = self.global_ns.free_function("fix_numeric")
        if 32 == self.architecture:
            if "0.9" in utils.xml_generator:
                if platform.machine() == "x86_64":
                    self.assertEqual(
                        fix_numeric.arguments[0].default_value, "-1u")
                else:
                    val = "0xffffffffffffffffu"
                    self.assertEqual(
                        fix_numeric.arguments[0].default_value, val)
            else:
                val = "0xffffffffffffffff"
                self.assertEqual(
                    fix_numeric.arguments[0].default_value, val)
        else:
            if "CastXML" in utils.xml_generator:
                if utils.xml_output_version >= 1.137:
                    val = "(unsigned long long)-1"
                else:
                    val = "(ull)-1"
                self.assertEqual(
                    fix_numeric.arguments[0].default_value, val)
            else:
                self.assertEqual(
                    fix_numeric.arguments[0].default_value, "0ffffffff")

    def test_unqualified_integral_patcher(self):
        if 32 != self.architecture and "CastXML" not in utils.xml_generator:
            # For this check to be removed, patcher_tester_64bit.xml
            # will need to be updated for GCCXML
            return

        ns1 = self.global_ns.namespace("ns1")
        st1 = ns1.class_("st1")
        fun1 = st1.member_function("fun1")
        if "CastXML" in utils.xml_generator:
            if utils.xml_output_version >= 1.137:
                val1 = "ns1::DEFAULT_1"
                val2 = "ns1::st1::DEFAULT_2"
            else:
                val1 = "::ns1::DEFAULT_1"
                val2 = "::ns1::st1::DEFAULT_2"
            self.assertEqual(
                fun1.arguments[0].default_value, val1)
            self.assertEqual(
                fun1.arguments[1].default_value, val2)
        else:
            self.assertEqual(
                fun1.arguments[0].default_value,
                "ns1::DEFAULT_1")
            self.assertEqual(
                fun1.arguments[1].default_value,
                "ns1::st1::DEFAULT_2")

        fun2 = self.global_ns.free_fun("fun2")
        self.assertEqual(
            fun2.arguments[0].default_value,
            "::DEFAULT_1")
        if "CastXML" in utils.xml_generator:
            if utils.xml_output_version >= 1.137:
                val1 = "ns1::DEFAULT_1"
                val2 = "ns1::st1::DEFAULT_2"
            else:
                # Before XML output version 1.137, the following two
                # were unpatched and were identical to the text in
                # matcher.hpp
                val1 = "ns1::DEFAULT_1"
                val2 = "::ns1::st1::DEFAULT_2"
            self.assertEqual(
                fun2.arguments[1].default_value, val1)
            self.assertEqual(
                fun2.arguments[2].default_value, val2)
        else:
            self.assertEqual(
                fun2.arguments[1].default_value,
                "ns1::DEFAULT_1")
            self.assertEqual(
                fun2.arguments[2].default_value,
                "ns1::st1::DEFAULT_2")

    def test_unnamed_enum_patcher(self):
        fix_unnamed = self.global_ns.free_fun("fix_unnamed")
        self.assertEqual(
            fix_unnamed.arguments[0].default_value, "int(::fx::unnamed)")

    def test_function_call_patcher(self):
        fix_function_call = self.global_ns.free_fun("fix_function_call")
        default_val = fix_function_call.arguments[0].default_value
        if "CastXML" in utils.xml_generator:
            if utils.xml_output_version >= 1.137:
                val = "function_call::calc(1, 2, 3)"
            else:
                val = "calc(1, 2, 3)"
        elif "0.9" in utils.xml_generator:
            val = "function_call::calc(1, 2, 3)"
        else:
            val = "function_call::calc( 1, 2, 3 )"
        self.assertEqual(default_val, val)

    def test_fundamental_patcher(self):
        fcall = self.global_ns.free_fun("fix_fundamental")
        if self.__cxx_std.is_cxx11_or_greater:
            val = "(unsigned int)(::fundamental::spam::eggs)"
        else:
            val = "(unsigned int)(::fundamental::eggs)"
        self.assertEqual(
            fcall.arguments[0].default_value, val)

    def test_constructor_patcher(self):
        typedef__func = self.global_ns.free_fun("typedef__func")
        default_val = typedef__func.arguments[0].default_value
        if "0.9" in utils.xml_generator:
            val = "typedef_::original_name()"
        elif "CastXML" in utils.xml_generator:
            # Most clean output, no need to patch
            val = "typedef_::alias()"
        else:
            val = "::typedef_::alias( )"
        self.assertEqual(default_val, val)
        if 32 == self.architecture:
            clone_tree = self.global_ns.free_fun("clone_tree")
            default_values = []
            if "0.9" in utils.xml_generator:
                default_values = []
            else:
                default_values = [
                    ("vector<std::basic_string<char, std::char_traits<char>," +
                        " std::allocator<char> >,std::allocator" +
                        "<std::basic_string<char, std::char_traits<char>, " +
                        "std::allocator<char> > > >()"),
                    ("vector<std::basic_string<char, std::char_traits<char>," +
                        "std::allocator<char> >,std::allocator" +
                        "<std::basic_string<char, std::char_traits<char>, " +
                        "std::allocator<char> > > >((&allocator" +
                        "<std::basic_string<char, std::char_traits<char>, " +
                        "std::allocator<char> > >()))")]
                self.assertIn(
                    clone_tree.arguments[0].default_value in default_values)


class tester_32_t(tester_impl_t):
    global_ns = None

    def __init__(self, *args):
        tester_impl_t.__init__(self, 32, *args)

    def setUp(self):
        if not tester_32_t.global_ns:
            reader = parser.source_reader_t(self.config)
            tester_32_t.global_ns = reader.read_file(
                "patcher.hpp")[0].top_parent
        self.global_ns = tester_32_t.global_ns


class tester_64_t(tester_impl_t):
    global_ns = None

    def __init__(self, *args):
        tester_impl_t.__init__(self, 64, *args)
        self.original_get_architecture = utils.get_architecture

    def setUp(self):
        self.original_get_architecture = utils.get_architecture
        utils.get_architecture = lambda: 64

        if not tester_64_t.global_ns:
            reader = parser.source_reader_t(self.config)
            if "castxml" not in self.config.xml_generator:
                tester_64_t.global_ns = reader.read_xml_file(
                    os.path.join(
                        autoconfig.data_directory,
                        "patcher_tester_64bit.xml"))[0].top_parent
            else:
                tester_64_t.global_ns = reader.read_file(
                    "patcher.hpp")[0].top_parent
        self.global_ns = tester_64_t.global_ns

    def tearDown(self):
        utils.get_architecture = self.original_get_architecture


def create_suite():
    suite = unittest.TestSuite()
    if "castxml" not in autoconfig.cxx_parsers_cfg.gccxml.xml_generator:
        suite.addTest(unittest.makeSuite(tester_32_t))
    suite.addTest(unittest.makeSuite(tester_64_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'vector_traits.hpp'
        self.global_ns = None

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
        self.global_ns = Test.global_ns

    def validate_yes(self, value_type, container):
        traits = declarations.vector_traits
        self.assertTrue(traits.is_my_case(container))
        self.assertTrue(
            declarations.is_same(
                value_type,
                traits.element_type(container)))
        self.assertTrue(traits.is_sequence(container))

    def test_global_ns(self):
        value_type = self.global_ns.class_('_0_')
        container = self.global_ns.typedef('container', recursive=False)
        self.validate_yes(value_type, container)

    def test_yes(self):
        yes_ns = self.global_ns.namespace('yes')
        for struct in yes_ns.classes():
            if not struct.name.startswith('_'):
                continue
            if not struct.name.endswith('_'):
                continue
            self.validate_yes(
                struct.typedef('value_type'),
                struct.typedef('container'))

    def test_no(self):
        traits = declarations.vector_traits
        no_ns = self.global_ns.namespace('no')
        for struct in no_ns.classes():
            if not struct.name.startswith('_'):
                continue
            if not struct.name.endswith('_'):
                continue
            self.assertTrue(not traits.is_my_case(struct.typedef('container')))

    def test_declaration(self):
        cnt = (
            'std::vector<std::basic_string<char, std::char_traits<char>, ' +
            'std::allocator<char> >,std::allocator<std::basic_string<char, ' +
            'std::char_traits<char>, std::allocator<char> > > >' +
            '@::std::vector<std::basic_string<char, std::char_traits<char>, ' +
            'std::allocator<char> >,std::allocator<std::basic_string<char, ' +
            'std::char_traits<char>, std::allocator<char> > > >')
        traits = declarations.find_container_traits(cnt)
        self.assertTrue(declarations.vector_traits is traits)

    def test_element_type(self):
        do_nothing = self.global_ns.free_fun('do_nothing')
        v = declarations.remove_reference(
            declarations.remove_declarated(
                do_nothing.arguments[0].decl_type))
        declarations.vector_traits.element_type(v)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'free_operators.hpp'
        self.global_ns = None

    def setUp(self):
        reader = parser.source_reader_t(self.config)
        decls = reader.read_file(self.header)
        self.global_ns = declarations.get_global_namespace(decls)

    def test(self):
        fo = self.global_ns.namespace('free_operators')
        number = fo.class_('number')
        rational = fo.class_('rational')
        for oper in fo.free_operators():
            if number.name in str(oper):
                self.assertTrue(number in oper.class_types)
            if rational.name in str(oper):
                self.assertTrue(rational in oper.class_types)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'declarations_calldef.hpp'
        self.declarations = None

    def setUp(self):
        if not self.declarations:
            self.declarations = parser.parse([self.header], self.config)

    def test(self):
        criteria = declarations.calldef_matcher_t(
            name='return_default_args',
            return_type='int',
            arg_types=[None, declarations.bool_t()])
        rda = declarations.matcher.get_single(criteria, self.declarations)
        self.assertTrue(rda, "return_default_args function was not found.")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import declarations


class tester_t(parser_test_case.parser_test_case_t):

    def __test_split_impl(self, decl_string, name, args):
        self.assertTrue(
            (name, args) == declarations.call_invocation.split(decl_string))

    def __test_split_recursive_impl(self, decl_string, control_seq):
        self.assertTrue(
            control_seq ==
            declarations.call_invocation.split_recursive(decl_string))

    def __test_is_call_invocation_impl(self, decl_string):
        self.assertTrue(
            declarations.call_invocation.is_call_invocation(decl_string))

    def test_split_on_vector(self):
        self.__test_is_call_invocation_impl("vector(int,std::allocator(int) )")

        self.__test_split_impl(
            "vector(int,std::allocator(int) )",
            "vector",
            ["int", "std::allocator(int)"])

        self.__test_split_recursive_impl(
            "vector(int,std::allocator(int) )",
            [("vector", ["int", "std::allocator(int)"]),
                ("std::allocator", ["int"])])

    def test_split_on_string(self):
        self.__test_is_call_invocation_impl(
            "basic_string(char,std::char_traits(char),std::allocator(char) )")

        self.__test_split_impl(
            "basic_string(char,std::char_traits(char),std::allocator(char) )",
            "basic_string",
            ["char", "std::char_traits(char)", "std::allocator(char)"])

    def test_split_on_map(self):
        self.__test_is_call_invocation_impl(
            "map(long int,std::vector(int, std::allocator(int) )," +
            "std::less(long int),std::allocator(std::pair" +
            "(const long int, std::vector(int, std::allocator(int) ) ) ) )")

        self.__test_split_impl(
            "map(long int,std::vector(int, std::allocator(int) )," +
            "std::less(long int),std::allocator(std::pair" +
            "(const long int, std::vector(int, std::allocator(int) ) ) ) )",
            "map",
            ["long int", "std::vector(int, std::allocator(int) )",
                "std::less(long int)",
                "std::allocator(std::pair(const long int," +
                " std::vector(int, std::allocator(int) ) ) )"])

    def test_join_on_vector(self):
        self.assertTrue(
            "vector( int, std::allocator(int) )" ==
            declarations.call_invocation.join(
                "vector", ("int", "std::allocator(int)")))

    def test_find_args(self):
        temp = 'x()()'
        found = declarations.call_invocation.find_args(temp)
        self.assertTrue((1, 2) == found)
        found = declarations.call_invocation.find_args(temp, found[1] + 1)
        self.assertTrue((3, 4) == found)
        temp = 'x(int,int)(1,2)'
        found = declarations.call_invocation.find_args(temp)
        self.assertTrue((1, 9) == found)
        found = declarations.call_invocation.find_args(temp, found[1] + 1)
        self.assertTrue((10, 14) == found)

    def test_bug_unmatched_brace(self):
        src = 'AlternativeName((&string("")), (&string("")), (&string("")))'
        self.__test_split_impl(
            src, 'AlternativeName', [
                '(&string(""))', '(&string(""))', '(&string(""))'])


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(tester_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2009 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "test_function_pointer.hpp"

    def test_function_pointer(self):
        """
        Test working with pointers and function pointers.

        """

        decls = parser.parse([self.header], self.config)
        global_ns = declarations.get_global_namespace(decls)

        # Test on a function pointer
        criteria = declarations.variable_matcher(name="func1")
        variables = declarations.matcher.find(criteria, global_ns)

        self.assertTrue(variables[0].name == "func1")
        self.assertTrue(
            isinstance(variables[0].decl_type, declarations.pointer_t))
        self.assertTrue(
            str(variables[0].decl_type) == "void (*)( int,double ) *")
        self.assertTrue(
            declarations.is_calldef_pointer(variables[0].decl_type))
        self.assertTrue(
            isinstance(declarations.remove_pointer(variables[0].decl_type),
                       declarations.free_function_type_t))

        # Get the function (free_function_type_t) and test the return and
        # argument types
        function = variables[0].decl_type.base
        self.assertTrue(isinstance(function.return_type, declarations.void_t))
        self.assertTrue(
            isinstance(function.arguments_types[0], declarations.int_t))
        self.assertTrue(
            isinstance(function.arguments_types[1], declarations.double_t))

        # Test on a normal pointer
        criteria = declarations.variable_matcher(name="myPointer")
        variables = declarations.matcher.find(criteria, global_ns)

        self.assertTrue(variables[0].name == "myPointer")
        self.assertTrue(
            isinstance(variables[0].decl_type, declarations.pointer_t))
        self.assertFalse(
            declarations.is_calldef_pointer(variables[0].decl_type))
        self.assertTrue(
            isinstance(declarations.remove_pointer(variables[0].decl_type),
                       declarations.volatile_t))

        # Test on function pointer in struct (x8)
        for d in global_ns.declarations:
            if d.name == "x8":
                self.assertTrue(
                    isinstance(d.decl_type, declarations.pointer_t))
                self.assertTrue(declarations.is_calldef_pointer(d.decl_type))
                self.assertTrue(
                    isinstance(
                        declarations.remove_pointer(d.decl_type),
                        declarations.member_function_type_t))
                self.assertTrue(
                    str(declarations.remove_pointer(d.decl_type)) ==
                    "void ( ::some_struct_t::* )(  )")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import autoconfig
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = os.path.join(
            autoconfig.data_directory,
            'declarations_calldef.hpp')
        self.template = """
        //test generated declaration string using gcc(xml) compiler
        #include "declarations_calldef.hpp"
        void test_generated_decl_string( %s );
        """

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def test_member_function(self):
        member_inline_call = self.global_ns.mem_fun('member_inline_call')
        decls = parser.parse_string(
            self.template %
            member_inline_call.decl_string,
            self.config)
        self.assertTrue(
            decls,
            "Created decl_string for member function contains mistake")

    def test_free_function(self):
        return_default_args = self.global_ns.free_fun('return_default_args')
        decls = parser.parse_string(
            self.template %
            return_default_args.decl_string,
            self.config)
        self.assertTrue(
            decls,
            "Created decl_string for global function contains mistake")

    def test_all_mem_and_free_funs(self):
        ns = self.global_ns.namespace('::declarations::calldef')
        for f in ns.mem_funs():
            decls = parser.parse_string(
                self.template % f.decl_string, self.config)
            self.assertTrue(
                decls,
                "Created decl_string for member function contains mistake")
        for f in ns.free_funs():
            decls = parser.parse_string(
                self.template % f.decl_string, self.config)
            self.assertTrue(
                decls,
                "Created decl_string for member function contains mistake")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __test_split_impl(self, decl_string, name, args):
        self.assertTrue(
            (name, args) == declarations.templates.split(decl_string))

    def __test_split_recursive_impl(self, decl_string, control_seq):
        self.assertTrue(
            control_seq == declarations.templates.split_recursive(decl_string))

    def __test_is_template_impl(self, decl_string):
        self.assertTrue(declarations.templates.is_instantiation(decl_string))

    def test_split_on_vector(self):
        self.__test_is_template_impl("vector<int,std::allocator<int> >")

        self.__test_split_impl(
            "vector<int,std::allocator<int> >",
            "vector",
            ["int", "std::allocator<int>"])

        self.__test_split_recursive_impl(
            "vector<int,std::allocator<int> >",
            [("vector", ["int", "std::allocator<int>"]),
                ("std::allocator", ["int"])])

    def test_split_on_string(self):
        self.__test_is_template_impl(
            "basic_string<char,std::char_traits<char>,std::allocator<char> >")

        self.__test_split_impl(
            "basic_string<char,std::char_traits<char>,std::allocator<char> >",
            "basic_string",
            ["char",
             "std::char_traits<char>",
             "std::allocator<char>"])

    def test_split_on_map(self):
        self.__test_is_template_impl(
            "map<long int,std::vector<int, std::allocator<int> >," +
            "std::less<long int>,std::allocator<std::pair<const long int, " +
            "std::vector<int, std::allocator<int> > > > >")

        self.__test_split_impl(
            "map<long int,std::vector<int, std::allocator<int> >," +
            "std::less<long int>,std::allocator<std::pair<const long int, " +
            "std::vector<int, std::allocator<int> > > > >",
            "map",
            ["long int",
             "std::vector<int, std::allocator<int> >",
             "std::less<long int>",
             "std::allocator<std::pair<const long int, " +
             "std::vector<int, std::allocator<int> > > >"])

    def test_join_on_vector(self):
        self.assertTrue(
            "vector< int, std::allocator<int> >" ==
            declarations.templates.join(
                "vector", ("int", "std::allocator<int>")))

    def test_bug_is_tmpl_inst(self):
        self.assertTrue(
            declarations.templates.is_instantiation(
                "::FX::QMemArray<unsigned char>::setRawData") is False)

    # disable broken test
    # def test_split_bug_fptr(self):
    #     x = 'map<std::string, bool (*)(std::string&, ' +
    #     'Ogre::MaterialScriptContext&), std::less<std::string>, ' +
    #     'std::allocator<std::pair<std::string const, bool (*)' +
    #     '(std::string&, Ogre::MaterialScriptContext&)> > >'
    #     name, args = declarations.templates.split( x )
    #     self.assertTrue( len(x) == 4, "This test is expected to fail." )


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()
